#ifndef LEXR_HH
#define LEXR_HH

#include <iosfwd>
#include <string>
#include "Translate.hh"

class Lexr {
public:

  /** Lexographic type codes
   */
  enum LxTags {
    tkEnd,           //  End of input
    tkSymbol,        //  Symbol name
    tkLiteral,       //  Numeric value
    tkString,        //  Quote delimited string.
    tkOper,          //  Operator string
    tkOParen,        //  Open parenthesis  ("(" or "{" in token string)
    tkCParen,        //  Close parenthesis (")" or "}" in token string)
    tkOSubs,         //  Open parenthesis  ("[" or "<" in token string)
    tkCSubs,         //  Close parenthesis ("]" or "<" in token string)
    tkComma          //  Delimiter         ("," or ":" in token string)
  };

private:
  /** Character type enumerator
   */
  enum ChartEnum {
      kInvChar,      //  Invalid character
      kAlpha,        //  Alphabet a-z, A-Z and '_'
      kNumer,        //  Number 0-9
      kQuote,        //  ' or "
      kParen,        //  Parentheses ()[]
      kOper,         //  Operator "+-*/=<>&|!%"
      kComma,        //  Delimiter ",:"
      kDot,          //  Period (used in numbers)
      kSpace         //  White space (ignored - " \t\n")
  };

  enum StateID {
      start,         //  nothing read yet
      in_symbol,     //  reading a symbol
      in_string,     //  reading a quoted string
      in_oper,       //  reading an operator string.
      numer_int,     //  reading the integer part of a number
      numer_fix,     //  reading a decimal fraction.
      numer_exp,     //  found nnnne
      numer_sexp     //  found nnnne[+- ][n]
  };

public:
  Lexr(void);
  virtual ~Lexr(void) {}

  /**  Perform a lexographical analysis of the specified input stream.
   */
  virtual LxTags getToken(std::istream& in, std::string& token) const;

  void setDebug(int lvl);

  void addDelimChars(const char* str);
  void addOpChars(const char* str);
  void addParenChars(const char* str);

  void setDelimChars(const char* str);
  void setOpChars(const char* str);
  void setParenChars(const char* str);

private:
  Translate<ChartEnum> mTrTable;
  int mDebug;
};

#endif // LEXR_HH
