#include <detect.h>
#include <math.h>

/*--------------------------------------------------------------------------*
 * Function: BOOL FfirBB(detector,points,lookback,data,coeff,coefNum)    *
 *--------------------------------------------------------------------------*
 *   Arguments:   struct detect_info *detector - context info               *
 *                WORD points - number of data points in the current record *
 *                WORD lookback - amount of lookback required               *
 *                                  for this filter                         *
 *                LONG *data - array that contains the data points          *
 *                                  to be processed by this FIR filter      *
 *--------------------------------------------------------------------------*
 *   Returns:     Returns TRUE if job completed successfully                *
 *                    (not used in this implementation)                     *
 *--------------------------------------------------------------------------*
 *   Lookback:    This filter has a lookback passed from "PTChan"           *
 *--------------------------------------------------------------------------*
 *   This routine processes the current record, which has "points" that are *
 *   data values from seismic or laser channels, with an FIR filter.  The   *
 *   array "data" contains these sample points, plus the last "lookback"    *
 *   values from the previous record.  (The values from the previous record *
 *   are stored in the 0th through lookback-1 positions of the array.)      *
 *   Hence, upon input, the first sample of the current record is stored at *
 *   the index "lookback" of the array.                                     *
 *                                                                          *
 *   To conserve RAM, we use the same array (the array "data") for both     *
 *   input to the filter and output of it:  Each time we process a new      *
 *   sample, a slot is free above the filtered data;  hence when the record *
 *   has been completely processed, the first "points" slots of the array   *
 *   contain the filtered seismic or laser samples of the record.           *
 *--------------------------------------------------------------------------*/

BOOL FfirBB(detector, point, lookback, data, coeff,coefNum)
struct detect_info *detector;
WORD point, lookback;
LONG *data;
double * coeff;
LONG coefNum; 

{
  LONG *ldp;

  double fresult = 0.;
  WORD i, j;

  data = detector->indatar;
  coeff = detector->coefPtr; 

  ldp = &data[lookback];
    /* the address of the first sample of the current record */

  for (i=0; i < point; i++) {
    /* process all of the data of the current record */
  
    for (j=0; j < coefNum; j++){
         /* Carry out convolution */

     fresult += coeff[j] *(double)(*(ldp - (LONG)j));
    }
      data[i] = rounding(fresult); 
      fresult = 0.;
      ldp++;                 /* address of next sample */
         /* store the result in the slot just above */
         /* the lookback of the next sample         */
  }

  return(TRUE);
}

LONG  rounding(double x)
{
   int a;
   x=x*1000;
   if (x - floor(x) >= 0.5) {
      a = (long) ceil(x);
    } else {
      a = (long) floor(x);
    }
  return (a);
  }
