 #include <sys/types.h>
#include <pthread.h>
#include <unistd.h>
#include <iostream>
#include <TTimer.h>
#include <TCanvas.h>
#include <TROOT.h>
#include <TEnv.h>
#include <TApplication.h>
#include "SpecMon.hh"
#include "HistogramUtil.hh"

#include <TPaveText.h>
#include <TSocket.h>
#include <TServerSocket.h>
#include <TMessage.h>
#include <TMonitor.h>
#include <TRandom.h>
#include <vector>
#include <string>
#include <TGNumberEntry.h>
#include <TGraph.h>

#include <iomanip>
#include <fstream>
#include <string>
#include <stdexcept>
#include <vector>
#include <stdlib.h>
#include <math.h>
#include "Dacc.hh"
#include "DatEnv.hh"
#include "TrigRslt.hh"
#include "TrigBase.hh"

#include <TStyle.h>

#include <TSeries.hh>
#include <FSeries.hh>



#define PIDCVSHDR "$Header: /ldcg_server/common/repository_gds/gds/Monitors/SpecMon/SpecMon.cc,v 1.1 2005/06/08 19:56:24 bodtim Exp $"
#define PIDTITLE "SpecMon "
#include "ProcIdent.hh"

// Monitor name
const char* const kMonName = "Spectral Power Monitor";
// Heartbeat rate
const Long_t kHeartbeatInterval = 300000; // 100 ms


//Channel Selection button widget id
const Int_t kChannelSelect = -1000;

//Band Selection button widget id
const Int_t kBandSelect = -2000;

//sub id for Idiots
const Int_t kBandSelectIdiot = -2001;

const Int_t kButtonExit = -3000;
const Int_t kButtonReset = -4000;
const Int_t kButtonUpdate = -5000;
const Int_t kButtonLogScale = -6000;
const Int_t kButtonUpdateScale = -7000;
const Int_t kButtonBins = -8000;
const Int_t kFreqHi = -9000;
const Int_t kFreqLo = -10000;
const Int_t kPowerLo = -11000;
const Int_t kPowerHi = -12000;
const Int_t kButtonLogScaleX = -13000;
const Int_t kButtonLogScaleStrip = -14000;
const Int_t kButtonLogScaleStripX = -15000;
const Int_t kTestBand = 9999;//id number for process message using test band


// Pause button widget id
const Int_t kButtonPause = 2;
// Resume button widget id
const Int_t kButtonResume = 3;

// Channel selection widget id
const Int_t kChnSel = 10;
// X min selection
const Int_t kXLimitMin = 11;
// X min selection
const Int_t kXLimitMax = 12;
// X bin selection
const Int_t kXBins = 13;

const char* const kButtonText[] = 
  {"      &Reset      ", "      &Pause      ", 
   "      R&esume      ", "       E&xit       "};
const char* const kButtonHelp[] = 
  {"Reset the histogram",
   "Pause data processing", 
   "Resume data processing", 
   "Exit program"};
///////////////////////////
string IFO;//////////////// this is so that the gui can know which part of config file to read without having to hardcode it
///////////////////////////




/// Create main program
EXEROOT (SpecMon);

using namespace std;
using namespace ligogui;

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// GuiMonConf                                                           //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
GuiMonConf::GuiMonConf () 
  : fNew (false), fValid (false), fPause (false), 
    fChnName ("H2:LSC-AS_Q") 
{
  fXLimit[0] = -32000.0; 
  fXLimit[1] = +32000.0; 
  fXBins = 100;
}
//////////////////////////////////////////////////////////////////////////
//                                                                      //
// GuiMonData                                                           //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
GuiMonData::GuiMonData () 
  : fNew (false) 
{
}

//______________________________________________________________________________
GuiMonData::~GuiMonData () 
  
{
}


//////////////////////////////////////////////////////////////////////////
//                                                                      //
// SpecMonWindow                                                         //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
GContext_t SpecMonWindow::fgButtonGC = 0;
FontStruct_t SpecMonWindow::fgButtonFont = 0;

//______________________________________________________________________________
SpecMonWindow::SpecMonWindow (const TGWindow *p, UInt_t w, UInt_t h,
			      SpecMon& monitor, const char* chnnames)
  : TGMainFrame (p, w, h), fMonitor (&monitor), fConf (&monitor.fConf), 
    fData (&monitor.fData),BandTestingMode(false),fSkipHeartbeats (0),fSocketUse(false)
  
{

  logscale = false;
  bins = false;  
  vector<string> info = readconfig("SpecMon.conf",IFO);//should have mConfigName,mIFOname instead change later
  int infoposition = 0;
  
  //loop for debug purposes to see what comes out of the config file
  //add debug statement later
  if(mDebug == mDebugAll)
    {
      for(size_t i=0;i<info.size();++i)
	{
	  cout<<info[infoposition]<<endl;
	  ++infoposition;
	}
    }  
  infoposition =0;//reset what position you will iterate at so that the config info is put into Channel and BandInfo structures
  
  mChannelNumber = atoi(info[0].c_str());
  ++infoposition;
  
  if(mDebug == mDebugConfigfile || mDebug == mDebugAll)
    {
      cout<<"mChannelNumber = "<<mChannelNumber<<endl;
    }
  
  //_______________________CHANNEL LOOP__________________________
	
  for(int i=0;i<mChannelNumber;++i)//channel loop
    {
      ChannelInfo Channel_temp;
      Channel_temp.ChannelName = info[infoposition];
      if(mDebug == mDebugConfigfile || mDebug == mDebugAll)
	{
	  cout<<"Channel Name is "<<Channel_temp.ChannelName<<endl;
	}
      ++infoposition;   
      
      Channel_temp.BandNum = atoi(info[infoposition].c_str());
      ++infoposition;
       
      for(int a=0;a<Channel_temp.BandNum;++a)//band loop
	{
	  BandInfo Band;
	  
	  Band.BandName = info[infoposition];
	  ++infoposition;
	  
	  Band.freqlo = atof(info[infoposition].c_str());
	  ++infoposition;
	  
	  Band.freqhi = atof(info[infoposition].c_str());
	  ++infoposition;
	  
	  Band.powerlo = atof(info[infoposition].c_str());
	  ++infoposition;
	  
	  Band.powerhi = atof(info[infoposition].c_str());
	  ++infoposition;
	  
	  Channel_temp.Bands.insert(Channel_temp.Bands.end(),Band);  
	}
           
      Channels.insert(Channels.end(),Channel_temp);
      Channel_temp.Bands.clear();
    }

  /********************
   *    	      *
   *    LayoutHints   *
   *		      *
   ********************/
  fL[0] = new TGLayoutHints (kLHintsLeft | kLHintsTop | kLHintsExpandX | kLHintsExpandY, 0, 0, 0, 0);//(hints, padleft, padright, padtop, padbottom)
  fL[1] = new TGLayoutHints (kLHintsLeft | kLHintsTop, 0, 0, 0, 0);
  fL[2] = new TGLayoutHints (kLHintsLeft, 0, 0, 0, 0);
  fL[3] = new TGLayoutHints (kLHintsRight |  kLHintsExpandX, 0, 0, 0, 0);
  fL[4] = new TGLayoutHints (kLHintsLeft | kLHintsTop | kLHintsExpandX | kLHintsExpandY, 0, 0, 0, 0);
  fL[5] = new TGLayoutHints (kLHintsTop | kLHintsExpandY, 0, 0, 0, 0);
  fL[6] = new TGLayoutHints (kLHintsCenterX | kLHintsCenterY | kLHintsExpandY, 0, 0, 0, 0);
  fL[7] = new TGLayoutHints (kLHintsRight | kLHintsCenterY | kLHintsExpandY | kLHintsExpandX, 0, 0, 0, 0);
  fL[8] = new TGLayoutHints (kLHintsTop | kLHintsExpandX, 0, 0, 0, 0);
  fL[9] = new TGLayoutHints (kLHintsBottom | kLHintsExpandX | kLHintsExpandY, 0, 0, 0, 0);
  fL[10] = new TGLayoutHints (kLHintsRight,0,0,0,0);
  fL[11] = new TGLayoutHints (kLHintsTop | kLHintsCenterX | kLHintsExpandX,0,0,0,0);
  fL[12] = new TGLayoutHints(kLHintsLeft | kLHintsExpandX,0,0,0,0);
  fL[13] = new TGLayoutHints(kLHintsCenterX,0,0,0,5);
  fL[14] = new TGLayoutHints(kLHintsLeft | kLHintsExpandX | kLHintsExpandY,0,0,0,0);
  fL[15] = new TGLayoutHints(kLHintsLeft,5,5,5,5);
  fL[16] = new TGLayoutHints(kLHintsRight,5,5,5,5);
  fL[17] = new TGLayoutHints(kLHintsExpandX | kLHintsExpandY,0,0,5,5);
  fL[18] =  new TGLayoutHints(kLHintsLeft | kLHintsExpandY,5,5,5,5);
  fL[19] =  new TGLayoutHints(kLHintsRight | kLHintsExpandY,5,5,5,5);
  fL[20] = new TGLayoutHints(kLHintsLeft,35,0,0,0);
  fL[21] = new TGLayoutHints(kLHintsRight,0,45,0,0);
  fL[22] = new TGLayoutHints (kLHintsLeft, 5, 0, 0, 5);
  fL[23] = new TGLayoutHints (kLHintsLeft, 5, 0, 0, 0);
  fL[24] = new TGLayoutHints (kLHintsLeft | kLHintsExpandX, 0, 0, 0, 0);
  fL[25] = new TGLayoutHints (kLHintsLeft, 5, 0, 5, 5);

  // top frame fFrame[0]
  fFrame[0] = new TGCompositeFrame (this, w, h, kVerticalFrame);
  AddFrame(fFrame[0], fL[0]);

  //holds the selection boxes and the statistics fFrame[1]
  fFrame[1] = new TGCompositeFrame (fFrame[0], 10,10, kHorizontalFrame);
  fFrame[0]->AddFrame(fFrame[1],fL[11]);

  //selection boxes fFrame[2]
  fFrame[2] = new TGCompositeFrame (fFrame[1], 10,10, kVerticalFrame);
  fFrame[1]->AddFrame(fFrame[2],fL[1]);

  //channel selection fFrame[3]---------------------------------------------
  fFrame[3] = new TGCompositeFrame (fFrame[2], 10,10, kHorizontalFrame);
  fFrame[2]->AddFrame(fFrame[3],fL[25]);

  //channel selection label
  fLabel[0] = new TGLabel(fFrame[3],"Select Channel");
  fFrame[3]->AddFrame(fLabel[0],fL[2]);
  
  //channel selection combobox
  fChannelSelect = new TGComboBox(fFrame[3], kChannelSelect);//kChannelSelect is an identifier for the ProcessMessage
  
  fChannelSelect->AddEntry("Select a Channel",1000);
  for(int i =0; i<mChannelNumber;++i)
    {
      fChannelSelect->AddEntry(Channels[i].ChannelName.c_str(),i);//the i is to give the ProcessMessage the sub id of the choice
    }
  
  fChannelSelect->Select(1000);
  fChannelSelect->Associate(this);
  fChannelSelect->Resize(150,22);
  fFrame[3]->AddFrame(fChannelSelect,fL[23]);
  //end channel selection frame---------------------------------------------

  //band selection fFrame[4]------------------------------------------------
  fFrame[4] = new TGCompositeFrame (fFrame[2], 10,10, kHorizontalFrame);
  fFrame[2]->AddFrame(fFrame[4],fL[22]);

  //band  selection label
  fLabel[1] = new TGLabel(fFrame[4],"Select Band");
  fFrame[4]->AddFrame(fLabel[1],fL[2]);

  //band selcetion combobox
  /* Note
     When the gui is started up there is nothing contained in the combo box due to the fact no channel is selected
     selecting a channel calls the BandSelectUpdate() function.  For more info go to ProcessMessage
  */
  fBandSelect = new TGComboBox(fFrame[4],kBandSelect);
  fBandSelect->AddEntry("I usually pick a channel just for the heck of it",kBandSelectIdiot);
  fBandSelect->Select(0);
  fBandSelect->Associate(this);
  fBandSelect->Resize(150,22);
  fFrame[4]->AddFrame(fBandSelect,fL[23]);  
  NumberBandsToDelete = kBandSelectIdiot;//this tells how many entries there are in the BandSelect combobox which allows you to delete them through BandSelectUpdate()
  //end band selection frame-------------------------------------------------

  //freq selection labels fFrame[5]------------------------------------------
  fFrame[5] = new TGCompositeFrame (fFrame[2], 10,10, kHorizontalFrame);
  fFrame[2]->AddFrame(fFrame[5],fL[12]);  

  //labels for frequency boxes    
  fLabel[2] = new TGLabel(fFrame[5],"freqlo");
  fFrame[5]->AddFrame(fLabel[2],fL[20]);//maybe change the layouthints for these, i need sand,  ahhhhhhh
  
  fLabel[3] = new TGLabel(fFrame[5],"freqhi");
  fFrame[5]->AddFrame(fLabel[3],fL[21]);
  //end freq label frame-----------------------------------------------------

  //freq numeric entry areas fFrame[6]---------------------------------------
  fFrame[6] = new TGCompositeFrame (fFrame[2], 10,10, kHorizontalFrame);
  fFrame[2]->AddFrame(fFrame[6],fL[2]); 

  int freq[2];//pointer to the min and max allowable values of the frequecies
  
  freq[0] = -8192;
  freq[1] = 8192;
  
  //Frequency numeric entry areas
  fFreq[0] = new TLGNumericControlBox(fFrame[6],0.0,12,kFreqLo,kNESInteger,kNEAAnyNumber,kNELLimitMinMax,freq[0],freq[1]);//(frame,val,digits,id i think,look at TGNumberEntry)
  fFreq[0]->Associate(this);
  fFrame[6]->AddFrame(fFreq[0],fL[15]);
  
  fFreq[1] = new TLGNumericControlBox(fFrame[6],0.0,12,kFreqHi,kNESInteger,kNEAAnyNumber,kNELLimitMinMax,freq[0],freq[1]);
  fFreq[1]->Associate(this);
  fFrame[6]->AddFrame(fFreq[1],fL[16]);
  //end freq numberic entry areas---------------------------------------------

  //power selection labels fFrame[7]------------------------------------------
  fFrame[7] = new TGCompositeFrame (fFrame[2], 10,10, kHorizontalFrame);
  fFrame[2]->AddFrame(fFrame[7],fL[12]); 

  //labels for power comboboxes
  fLabel[4] = new TGLabel(fFrame[7],"hi power");
  fFrame[7]->AddFrame(fLabel[4],fL[21]);
  
  fLabel[5] = new TGLabel(fFrame[7],"lo power");
  fFrame[7]->AddFrame(fLabel[5],fL[20]);  
  //end power label frame-----------------------------------------------------

  //power numeric entry areas fFrame[8]---------------------------------------
  fFrame[8] = new TGCompositeFrame (fFrame[2], 10,10, kHorizontalFrame);
  fFrame[2]->AddFrame(fFrame[8],fL[2]);   

  double power[2];
  
  power[0] = 0.0;
  power[1] = 10000.0000;
  
  //Power numeric entry areas   
  fPower[1] = new TLGNumericControlBox(fFrame[8],0.0,12,kPowerHi,kNESInteger,kNEAPositive,kNELLimitMinMax,power[0],power[1]);//(frame,val,digits,id i think,look at TGNumberEntry)
  fPower[1]->Associate(this);
  fFrame[8]->AddFrame(fPower[1],fL[16]);
  
  fPower[0] = new TLGNumericControlBox(fFrame[8],0.0,12,kPowerLo,kNESInteger,kNEAPositive,kNELLimitMinMax,power[0],power[1]);
  fPower[0]->Associate(this);
  fFrame[8]->AddFrame(fPower[0],fL[15]);
  //end power numeric entry areas frame----------------------------------------
  
  //statistics fFrame[9]-------------------------------------------------------
  fFrame[9] = new TGCompositeFrame(fFrame[1], 10,10, kVerticalFrame);
  fFrame[1]->AddFrame(fFrame[9],fL[3]);  
  
  //frame for stats and buttons fFrame[20]
  fFrame[20] =  new TGCompositeFrame(fFrame[9], 10,10, kHorizontalFrame);
  fFrame[9]->AddFrame(fFrame[20],fL[12]);
   
  //frame for mean stats fFrame[21]
  fFrame[21] =  new TGCompositeFrame(fFrame[20], 10,10, kHorizontalFrame);
  fFrame[20]->AddFrame(fFrame[21],fL[11]);
  
  //holds mean,power, trigger hi, lo fFramenew[0]
  fFramenew[0] =  new TGCompositeFrame(fFrame[21], 10,10, kVerticalFrame);
  fFrame[21]->AddFrame(fFramenew[0],fL[17]);
  
  //mean label fFramenew[1]
  fFramenew[1] =  new TGCompositeFrame(fFramenew[0], 10,10, kVerticalFrame);
  fFramenew[0]->AddFrame(fFramenew[1],fL[17]);

  fLabelStats[0] = new TGLabel(fFramenew[1],"Mean");
  fFramenew[1]->AddFrame(fLabelStats[0],fL[17]);

  //power label fFramenew[2]
  fFramenew[2] =  new TGCompositeFrame(fFramenew[0], 10,10, kVerticalFrame);
  fFramenew[0]->AddFrame(fFramenew[2],fL[17]);

  fLabelStats[1] = new TGLabel(fFramenew[2],"Power");
  fFramenew[2]->AddFrame(fLabelStats[1],fL[17]);

  //trigger lo label fFramenew[3]
  fFramenew[3] =  new TGCompositeFrame(fFramenew[0], 10,10, kVerticalFrame);
  fFramenew[0]->AddFrame(fFramenew[3],fL[17]);

  fLabelStats[2] = new TGLabel(fFramenew[3],"Lo Trigger");
  fFramenew[3]->AddFrame(fLabelStats[2],fL[17]);

  //trigger hi label fFramenew[4]
  fFramenew[4] =  new TGCompositeFrame(fFramenew[0], 10,10, kVerticalFrame);
  fFramenew[0]->AddFrame(fFramenew[4],fL[17]);

  fLabelStats[3] = new TGLabel(fFramenew[4],"Hi Trigger");
  fFramenew[4]->AddFrame(fLabelStats[3],fL[17]);
  //--------------------------------------------------------------------

    //holds 1 min stuff
  fFramenew[25] =  new TGCompositeFrame(fFrame[21], 10,10, kVerticalFrame);
  fFrame[21]->AddFrame(fFramenew[25],fL[17]);

  //holds 1min label
  fFramenew[26] = new TGCompositeFrame(fFramenew[25],10,10,kVerticalFrame);
  fFramenew[25]->AddFrame(fFramenew[26],fL[17]);

  fLabelStats[20] = new TGLabel(fFramenew[26],"1 Min");
  fFramenew[26]->AddFrame(fLabelStats[20],fL[17]);

  //holds 1 min power
  fFramenew[27] = new TGCompositeFrame(fFramenew[25],10,10,kVerticalFrame);
  fFramenew[25]->AddFrame(fFramenew[27],fL[17]);

  fLabelStats[21] = new TGLabel(fFramenew[27],"");
  fFramenew[27]->AddFrame(fLabelStats[21],fL[17]);

  //holds 1 min trigger lo
  fFramenew[28] = new TGCompositeFrame(fFramenew[25],10,10,kVerticalFrame);
  fFramenew[25]->AddFrame(fFramenew[28],fL[17]);

  fLabelStats[22] = new TGLabel(fFramenew[28],"");
  fFramenew[28]->AddFrame(fLabelStats[22],fL[17]);

  //holds 1 min triggerhi
  fFramenew[29] = new TGCompositeFrame(fFramenew[25],10,10,kVerticalFrame);
  fFramenew[25]->AddFrame(fFramenew[29],fL[17]);

  fLabelStats[23] = new TGLabel(fFramenew[29],"");
  fFramenew[29]->AddFrame(fLabelStats[23],fL[17]);
  
  //----------------------------------
  
  //holds 10 min stuff
  fFramenew[30] =  new TGCompositeFrame(fFrame[21], 10,10, kVerticalFrame);
  fFrame[21]->AddFrame(fFramenew[30],fL[17]);
  
  //holds 10 min label
  fFramenew[31] = new TGCompositeFrame(fFramenew[30],10,10,kVerticalFrame);
  fFramenew[30]->AddFrame(fFramenew[31],fL[17]);

  fLabelStats[24] = new TGLabel(fFramenew[31],"10 Min");
  fFramenew[31]->AddFrame(fLabelStats[24],fL[17]);
  
  //holds 10 min power
  fFramenew[32] = new TGCompositeFrame(fFramenew[30],10,10,kVerticalFrame);
  fFramenew[30]->AddFrame(fFramenew[32],fL[17]);
  
  fLabelStats[25] = new TGLabel(fFramenew[32],"");
  fFramenew[32]->AddFrame(fLabelStats[25],fL[17]);
  
  //holds 10 min trigger lo
  fFramenew[33] = new TGCompositeFrame(fFramenew[30],10,10,kVerticalFrame);
  fFramenew[30]->AddFrame(fFramenew[33],fL[17]);

  fLabelStats[26] = new TGLabel(fFramenew[33],"");
  fFramenew[33]->AddFrame(fLabelStats[26],fL[17]);

  //holds 10 min trigger hi
  fFramenew[34] = new TGCompositeFrame(fFramenew[30],10,10,kVerticalFrame);
  fFramenew[30]->AddFrame(fFramenew[34],fL[17]);

  fLabelStats[27] = new TGLabel(fFramenew[34],"");
  fFramenew[34]->AddFrame(fLabelStats[27],fL[17]);
  
  //------------------------------------------------------------------------

  //holds 30 Min stuff
  fFramenew[35] =  new TGCompositeFrame(fFrame[21], 10,10, kVerticalFrame);
  fFrame[21]->AddFrame(fFramenew[35],fL[17]);

  //holds 30 min label
  fFramenew[36] = new TGCompositeFrame(fFramenew[35],10,10,kVerticalFrame);
  fFramenew[35]->AddFrame(fFramenew[36],fL[17]);

  fLabelStats[28] = new TGLabel(fFramenew[36],"30 Min");
  fFramenew[36]->AddFrame(fLabelStats[28],fL[17]);

  //holds 30 min power
  fFramenew[37] = new TGCompositeFrame(fFramenew[35],10,10,kVerticalFrame);
  fFramenew[35]->AddFrame(fFramenew[37],fL[17]);
  
  fLabelStats[29] = new TGLabel(fFramenew[37],"");
  fFramenew[37]->AddFrame(fLabelStats[29],fL[17]);

  //holds 30 min trigger lo
  fFramenew[38] = new TGCompositeFrame(fFramenew[35],10,10,kVerticalFrame);
  fFramenew[35]->AddFrame(fFramenew[38],fL[17]);

  fLabelStats[30] = new TGLabel(fFramenew[38],"");
  fFramenew[38]->AddFrame(fLabelStats[30],fL[17]);

  //holds 30 min trigger hi
  fFramenew[39] = new TGCompositeFrame(fFramenew[35],10,10,kVerticalFrame);
  fFramenew[35]->AddFrame(fFramenew[39],fL[17]);

  fLabelStats[31] = new TGLabel(fFramenew[39],"");
  fFramenew[39]->AddFrame(fLabelStats[31],fL[17]);

  //-------------------------------------------------------------------------
  
  //holds 1hr stuff fFramenew[5]
  fFramenew[5] =  new TGCompositeFrame(fFrame[21], 10,10, kVerticalFrame);
  fFrame[21]->AddFrame(fFramenew[5],fL[17]);

  //holds 1hr label fFramenew[6]
  fFramenew[6] = new TGCompositeFrame(fFramenew[5],10,10,kVerticalFrame);
  fFramenew[5]->AddFrame(fFramenew[6],fL[17]);

  fLabelStats[4] = new TGLabel(fFramenew[6],"1 Hr");
  fFramenew[6]->AddFrame(fLabelStats[4],fL[17]);

  //holds 1hr power fFramenew[7]
  fFramenew[7] = new TGCompositeFrame(fFramenew[5],10,10,kVerticalFrame);
  fFramenew[5]->AddFrame(fFramenew[7],fL[17]);

  fLabelStats[5] = new TGLabel(fFramenew[7],"");
  fFramenew[7]->AddFrame(fLabelStats[5],fL[17]);

  //holds 1hr triggerlo fFramenew[8]
  fFramenew[8] = new TGCompositeFrame(fFramenew[5],10,10,kVerticalFrame);
  fFramenew[5]->AddFrame(fFramenew[8],fL[17]);

  fLabelStats[6] = new TGLabel(fFramenew[8],"");
  fFramenew[8]->AddFrame(fLabelStats[6],fL[17]);

  //holds 1hr triggerhi fFramenew[9]
  fFramenew[9] = new TGCompositeFrame(fFramenew[5],10,10,kVerticalFrame);
  fFramenew[5]->AddFrame(fFramenew[9],fL[17]);

  fLabelStats[7] = new TGLabel(fFramenew[9],"");
  fFramenew[9]->AddFrame(fLabelStats[7],fL[17]);

  //----------------

  //holds 6hr stuff fFramenew[10]
  fFramenew[10] =  new TGCompositeFrame(fFrame[21], 10,10, kVerticalFrame);
  fFrame[21]->AddFrame(fFramenew[10],fL[17]);

  //holds 6hr label fFramenew[11]
  fFramenew[11] = new TGCompositeFrame(fFramenew[10],10,10,kVerticalFrame);
  fFramenew[10]->AddFrame(fFramenew[11],fL[17]);

  fLabelStats[8] = new TGLabel(fFramenew[11],"6 Hr");
  fFramenew[11]->AddFrame(fLabelStats[8],fL[17]);

  //holds 6hr power fFramenew[12]
  fFramenew[12] = new TGCompositeFrame(fFramenew[10],10,10,kVerticalFrame);
  fFramenew[10]->AddFrame(fFramenew[12],fL[17]);

  fLabelStats[9] = new TGLabel(fFramenew[12],"");
  fFramenew[12]->AddFrame(fLabelStats[9],fL[17]);

  //holds 6hr triggerlo fFramenew[13]
  fFramenew[13] = new TGCompositeFrame(fFramenew[10],10,10,kVerticalFrame);
  fFramenew[10]->AddFrame(fFramenew[13],fL[17]);

  fLabelStats[10] = new TGLabel(fFramenew[13],"");
  fFramenew[13]->AddFrame(fLabelStats[10],fL[17]);

  //holds 6hr triggerhi fFramenew[14]
  fFramenew[14] = new TGCompositeFrame(fFramenew[10],10,10,kVerticalFrame);
  fFramenew[10]->AddFrame(fFramenew[14],fL[17]);

  fLabelStats[11] = new TGLabel(fFramenew[14],"");
  fFramenew[14]->AddFrame(fLabelStats[11],fL[17]);

  //---------------------------

  //holds 12hr stuff fFramenew[15]
  fFramenew[15] =  new TGCompositeFrame(fFrame[21], 10,10, kVerticalFrame);
  fFrame[21]->AddFrame(fFramenew[15],fL[17]);

  //holds 12hr label fFramenew[16]
  fFramenew[16] = new TGCompositeFrame(fFramenew[15],10,10,kVerticalFrame);
  fFramenew[15]->AddFrame(fFramenew[16],fL[17]);

  fLabelStats[12] = new TGLabel(fFramenew[16],"12 Hr");
  fFramenew[16]->AddFrame(fLabelStats[12],fL[17]);


  //holds 12hr power fFramenew[17]
  fFramenew[17] = new TGCompositeFrame(fFramenew[15],10,10,kVerticalFrame);
  fFramenew[15]->AddFrame(fFramenew[17],fL[17]);

  fLabelStats[13] = new TGLabel(fFramenew[17],"");
  fFramenew[17]->AddFrame(fLabelStats[13],fL[17]);

  //holds 12hr triggerlo fFramenew[18]
  fFramenew[18] = new TGCompositeFrame(fFramenew[15],10,10,kVerticalFrame);
  fFramenew[15]->AddFrame(fFramenew[18],fL[17]);

  fLabelStats[14] = new TGLabel(fFramenew[18],"");
  fFramenew[18]->AddFrame(fLabelStats[14],fL[17]);

  //holds 12hr triggerhi fFramenew[19]
  fFramenew[19] = new TGCompositeFrame(fFramenew[15],10,10,kVerticalFrame);
  fFramenew[15]->AddFrame(fFramenew[19],fL[17]);

  fLabelStats[15] = new TGLabel(fFramenew[19],"");
  fFramenew[19]->AddFrame(fLabelStats[15],fL[17]);

  //---------------------

   //holds 24hr stuff
  fFramenew[20] =  new TGCompositeFrame(fFrame[21], 10,10, kVerticalFrame);
  fFrame[21]->AddFrame(fFramenew[20],fL[17]);

  //holds 24hr label
  fFramenew[21] = new TGCompositeFrame(fFramenew[20],10,10,kVerticalFrame);
  fFramenew[20]->AddFrame(fFramenew[21],fL[17]);

  fLabelStats[16] = new TGLabel(fFramenew[21],"24 Hr");
  fFramenew[21]->AddFrame(fLabelStats[16],fL[17]);

  //holds 24hr power
  fFramenew[22] = new TGCompositeFrame(fFramenew[20],10,10,kVerticalFrame);
  fFramenew[20]->AddFrame(fFramenew[22],fL[17]);

  fLabelStats[17] = new TGLabel(fFramenew[22],"");
  fFramenew[22]->AddFrame(fLabelStats[17],fL[17]);

  //holds 24hr triggerlo
  fFramenew[23] = new TGCompositeFrame(fFramenew[20],10,10,kVerticalFrame);
  fFramenew[20]->AddFrame(fFramenew[23],fL[17]);

  fLabelStats[18] = new TGLabel(fFramenew[23],"");
  fFramenew[23]->AddFrame(fLabelStats[18],fL[17]);

  //holds 24hr triggerhi
  fFramenew[24] = new TGCompositeFrame(fFramenew[20],10,10,kVerticalFrame);
  fFramenew[20]->AddFrame(fFramenew[24],fL[17]);

  fLabelStats[19] = new TGLabel(fFramenew[24],"");
  fFramenew[24]->AddFrame(fLabelStats[19],fL[17]);

  //frame for buttons fFrame[22]
  fFrame[22] = new TGCompositeFrame(fFrame[20],10,10,kVerticalFrame);
  fFrame[20]->AddFrame(fFrame[22],fL[10]);
  
  //holds exit and update
  fFrame[23] = new TGCompositeFrame(fFrame[22],10,10,kHorizontalFrame);
  fFrame[22]->AddFrame(fFrame[23],fL[10]);

  fExit = new TGTextButton(fFrame[23],"Exit",kButtonExit);
  fExit->Associate(this);
  fFrame[23]->AddFrame(fExit,fL[13]);

  fUpdate = new TGTextButton(fFrame[23],"Update",kButtonUpdate);
  fUpdate->Associate(this);
  fFrame[23]->AddFrame(fUpdate,fL[16]);

  //holds strip and histo buttons
  fFrame[24] = new TGCompositeFrame(fFrame[22],10,10,kHorizontalFrame);
  fFrame[22]->AddFrame(fFrame[24],fL[9]);

  //hold strip chart stuff
  fFrame[25] = new TGCompositeFrame(fFrame[24],10,10,kVerticalFrame);
  fFrame[24]->AddFrame(fFrame[25],fL[18]);

  fLabel[18] = new TGLabel(fFrame[25],"StripChart");
  fFrame[25]->AddFrame(fLabel[18],fL[8]);
   
  fLogScaleStripY = new TGTextButton(fFrame[25],"Y Log Scale",kButtonLogScaleStrip);
  fLogScaleStripY->Associate(this);
  fFrame[25]->AddFrame(fLogScaleStripY,fL[15]);

  fLogScaleStripX = new TGTextButton(fFrame[25],"X Log Scale",kButtonLogScaleStripX);
  fLogScaleStripX->Associate(this);
  fFrame[25]->AddFrame(fLogScaleStripX,fL[15]);

  //holds histo stuff
  fFrame[26] = new TGCompositeFrame(fFrame[24],10,10,kVerticalFrame);
  fFrame[24]->AddFrame(fFrame[26],fL[19]);

  fLabel[17] = new TGLabel(fFrame[26],"Histogram");
  fFrame[26]->AddFrame(fLabel[17],fL[8]);
  
  fLogScale = new TGTextButton(fFrame[26],"Y Log Scale",kButtonLogScale);
  fLogScale->Associate(this);
  fFrame[26]->AddFrame(fLogScale,fL[15]);

  fLogScaleX = new TGTextButton(fFrame[26],"X Log Scale",kButtonLogScaleX);
  fLogScaleX->Associate(this);
  fFrame[26]->AddFrame(fLogScaleX,fL[15]);

  //label for bins
  fLabel[8] = new TGLabel(fFrame[26],"Histogram bins");
  fFrame[26]->AddFrame(fLabel[8],fL[15]);

  int lobin = 20;
  int hibin = 1000;

  fBins = new TLGNumericControlBox(fFrame[26],20,4,kButtonBins,kNESInteger,kNEAPositive,kNELLimitMinMax,lobin,hibin);
  fBins->Associate(this);
  fFrame[26]->AddFrame(fBins,fL[16]);

  //end of statistics frame---------------------------------------------------------------
  
  //holds the stripchart, histogram  fFrame[10]
  fFrame[10] = new TGCompositeFrame (fFrame[0], 10,10, kHorizontalFrame);
  fFrame[0]->AddFrame(fFrame[10],fL[4]);

  //strip chart fFrame[11]----------------------------------------------------------------------
  fFrame[11] = new TGCompositeFrame (fFrame[10], 10,10, kVerticalFrame);
  fFrame[10]->AddFrame(fFrame[11],fL[4]);

  fPlot[0] = new TRootEmbeddedCanvas ("stripchart", fFrame[11], 390, 290);
  fStripChart = new TCanvas("6 hour strip chart of power",10,10,fPlot[0]->GetCanvasWindowId());
  fStripChart->cd();
  fStripChart->Update();
  fPlot[0]->AdoptCanvas(fStripChart);
  fFrame[11]->AddFrame(fPlot[0],fL[4]);
  //end stripchart frame------------------------------------------------------------------------

  //misc buttons fFrame[12]
  fFrame[12] = new TGCompositeFrame (fFrame[10], 10,10, kVerticalFrame);
  fFrame[10]->AddFrame(fFrame[12],fL[5]);
  
  //button 1 fFrame[13]
  fFrame[13] = new TGCompositeFrame (fFrame[12], 10,10, kHorizontalFrame);
  fFrame[12]->AddFrame(fFrame[13],fL[6]);  
  
  //button 2 fFrame[14]
  fFrame[14] = new TGCompositeFrame (fFrame[12], 10,10, kHorizontalFrame);
  fFrame[12]->AddFrame(fFrame[14],fL[6]);   

  //button 3 fFrame[15]
  fFrame[15] = new TGCompositeFrame (fFrame[12], 10,10, kHorizontalFrame);
  fFrame[12]->AddFrame(fFrame[15],fL[6]);  
  
  //button 4 fFrame[16]
  fFrame[16] = new TGCompositeFrame (fFrame[12], 10,10, kHorizontalFrame);
  fFrame[12]->AddFrame(fFrame[16],fL[6]);  

  //histogram and related buttons fFrame[17]
  fFrame[17] = new TGCompositeFrame (fFrame[10], 10,10, kVerticalFrame);
  fFrame[10]->AddFrame(fFrame[17],fL[7]);  

  //buttons frame fFrame[18]
  fFrame[18] = new TGCompositeFrame (fFrame[17], 10,10, kHorizontalFrame);
  fFrame[17]->AddFrame(fFrame[18],fL[8]); 

  //histogram fFrame[19]--------------------------------------------------------------------------
  fFrame[19] = new TGCompositeFrame (fFrame[17], 10,10, kHorizontalFrame);
  fFrame[17]->AddFrame(fFrame[19],fL[9]);   

  fPlot[1] = new TRootEmbeddedCanvas("histogram",fFrame[17],390,290);
  fHistogram = new TCanvas("6 hr histogram of Power",10,10,fPlot[1]->GetCanvasWindowId());
  fHistogram->cd();
  fHistogram->Update();
  fPlot[1]->AdoptCanvas(fHistogram);
  fFrame[19]->AddFrame(fPlot[1],fL[4]);
  //end of histogram frame-------------------------------------------------------------------------
     
  // Setup graphics context for buttons
  if (fgButtonGC == 0) {
    fgButtonFont = gClient->GetFontByName 
      (gEnv->GetValue
       ("Gui.NormalFont",
	"-adobe-helvetica-bold-r-*-*-14-*-*-*-*-*-iso8859-1"));
    GCValues_t   gval;
    gval.fMask = kGCForeground | kGCFont;
    gval.fFont = gVirtualX->GetFontHandle (fgButtonFont);
    gval.fForeground = fgBlackPixel;
    fgButtonGC = 
      gVirtualX->CreateGC (gClient->GetRoot()->GetId(), &gval);
  }
 
  // Set window name
  SetWindowName (kMonName);
  SetIconName (kMonName);
  SetClassHints (kMonName, kMonName);
  
  // resize and map
  SetWMPosition (0,0);
  MapSubwindows();
  Resize (GetDefaultSize());
  MapWindow();

  // initialze heartbeat timer
  fHeartbeat = new TTimer (this, kHeartbeatInterval, kTRUE);
  fHeartbeat->TurnOn();
}

//______________________________________________________________________________
SpecMonWindow::~SpecMonWindow()
{
  // delete timer
  delete fHeartbeat;
  // delete widgets
  delete fPlot;
  delete fXLimit[0];
  delete fXLimit[1];
  for (int i = 0; i < 12; ++i) delete fLabel[i];
  for (int i = 0; i < 20; ++i) delete fButton[i];
  for (int i = 0; i < 25; ++i) delete fFrame[i];
  for (int i = 0; i < 12; ++i) delete fL[i];
}

//______________________________________________________________________________
void SpecMonWindow::CloseWindow()
{
  // get out
  fMonitor->fMainWindow = 0;
  fMonitor->finish();
  TGMainFrame::CloseWindow();
  delete this;
  pthread_exit(0);
}

//______________________________________________________________________________
Bool_t SpecMonWindow::ReadConf()
{
  fConf->lock();
  fConf->fXLimit[0] = fXLimit[0]->GetNumber();
  fConf->fXLimit[1] = fXLimit[1]->GetNumber();
  fConf->fXBins = fXBins->GetIntNumber();
  fConf->fNew = true;
  fConf->fValid = true;
  fConf->unlock();
  return kTRUE;
}

//______________________________________________________________________________
Bool_t SpecMonWindow::DrawNew()
{
  fPlot[0]->GetCanvas()->cd();
  fHist.Draw ("");
  fPlot[0]->GetCanvas()->Update();
  return kTRUE;
}

//__________________________________________________________________________________

void SpecMonWindow::GetStripChart(TMessage *mess)
{
  TH1 *stripchart = (TH1 *)mess->ReadObject(mess->GetClass());
  fStripChart->cd();
  
  stripchart->SetLineColor(0);
  stripchart->GetXaxis()->SetTitle("Time");
  stripchart->GetYaxis()->SetTitle("Integrated Power (counts^2)");
  stripchart->SetStats(false);                   //not sure if this should be set
  
  stripchart->DrawCopy("*H");
  fStripChart->Modified();
  fStripChart->Update();
  
  delete stripchart;
}//END GETSTRIPCHART

void SpecMonWindow::GetHistogram(TMessage *mess)
{
  TH1 *histogram = (TH1 *)mess->ReadObject(mess->GetClass());
  fHistogram->cd();
  histogram->SetLineColor(1);
  histogram->GetXaxis()->SetTitle("Integrated Power");
  histogram->GetYaxis()->SetTitle("# of Occurences at Power");
  histogram->SetStats(false);
  histogram->DrawCopy();
  fHistogram->Modified();
  fHistogram->Update();
  
  /*not sure i need this
    
  fHistogram->cd();
  histogram->DrawCopy();
  fHistogram->Modified();
  fHistogram->Update();
  histogram_bool = false;
  */
  delete histogram;
}//END GETHISTOGRAM

void SpecMonWindow::GetMeanValues(TMessage *mess)
{
  TH1* mean_histogram = (TH1*)mess->ReadObject(mess->GetClass());
  Channels[fChannelSelected].Bands[fBandSelected]._1hr_mean = mean_histogram->GetBinContent(1);
  Channels[fChannelSelected].Bands[fBandSelected]._6hr_mean = mean_histogram->GetBinContent(2);
  Channels[fChannelSelected].Bands[fBandSelected]._12hr_mean = mean_histogram->GetBinContent(3);
  Channels[fChannelSelected].Bands[fBandSelected]._24hr_mean = mean_histogram->GetBinContent(4);
  Channels[fChannelSelected].Bands[fBandSelected].trigger1_lo = mean_histogram->GetBinContent(5);
  Channels[fChannelSelected].Bands[fBandSelected].trigger6_lo = mean_histogram->GetBinContent(6);
  Channels[fChannelSelected].Bands[fBandSelected].trigger12_lo = mean_histogram->GetBinContent(7);
  Channels[fChannelSelected].Bands[fBandSelected].trigger24_lo = mean_histogram->GetBinContent(8);
  Channels[fChannelSelected].Bands[fBandSelected].trigger1_hi = mean_histogram->GetBinContent(9);
  Channels[fChannelSelected].Bands[fBandSelected].trigger6_hi = mean_histogram->GetBinContent(10);
  Channels[fChannelSelected].Bands[fBandSelected].trigger12_hi = mean_histogram->GetBinContent(11);
  Channels[fChannelSelected].Bands[fBandSelected].trigger24_hi = mean_histogram->GetBinContent(12);
  
  Channels[fChannelSelected].Bands[fBandSelected].trigger30min_hi = mean_histogram->GetBinContent(13);
  Channels[fChannelSelected].Bands[fBandSelected].trigger30min_lo = mean_histogram->GetBinContent(14);
  Channels[fChannelSelected].Bands[fBandSelected]._30min_mean = mean_histogram->GetBinContent(15);
  Channels[fChannelSelected].Bands[fBandSelected].trigger10min_hi = mean_histogram->GetBinContent(16);
  Channels[fChannelSelected].Bands[fBandSelected].trigger10min_lo = mean_histogram->GetBinContent(17);
  Channels[fChannelSelected].Bands[fBandSelected]._10min_mean = mean_histogram->GetBinContent(18);
  Channels[fChannelSelected].Bands[fBandSelected].trigger1min_hi = mean_histogram->GetBinContent(19);
  Channels[fChannelSelected].Bands[fBandSelected].trigger1min_lo = mean_histogram->GetBinContent(20);
  Channels[fChannelSelected].Bands[fBandSelected]._1min_mean = mean_histogram->GetBinContent(21);
  
  //these hold the values of the mean, trigger means as char arrays so they can be displayed in labels.  
  char one[10];
  char six[50];
  char twelve[50];
  char twenty[50];
  char onetl[50];
  char sixtl[50];
  char twelvetl[50];
  char twentytl[50];
  char oneth[50];
  char sixth[50];
  char twelveth[50];
  char twentyth[50];
  
  char onemin[50];
  char oneminl[50];
  char oneminh[50];
  char tenmin[50];
  char tenminl[50];
  char tenminh[50];
  char thirty[50];
  char thirtyl[50];
  char thirtyh[50];
	
  sprintf(one,"%f",Channels[fChannelSelected].Bands[fBandSelected]._1hr_mean);
  sprintf(six,"%f",Channels[fChannelSelected].Bands[fBandSelected]._6hr_mean);
  sprintf(twelve,"%f",Channels[fChannelSelected].Bands[fBandSelected]._12hr_mean);
  sprintf(twenty,"%f",Channels[fChannelSelected].Bands[fBandSelected]._24hr_mean);
  sprintf(onetl,"%f", Channels[fChannelSelected].Bands[fBandSelected].trigger1_lo);
  sprintf(sixtl,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger6_lo);
  sprintf(twelvetl,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger12_lo);
  sprintf(twentytl,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger24_lo);
  sprintf(oneth,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger1_hi);
  sprintf(sixth,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger6_hi);
  sprintf(twelveth,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger12_hi);
  sprintf(twentyth,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger24_hi);
  
  sprintf(onemin,"%f",Channels[fChannelSelected].Bands[fBandSelected]._1min_mean);
  sprintf(oneminl,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger1min_lo);
  sprintf(oneminh,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger1min_hi);
  
  sprintf(tenmin,"%f",Channels[fChannelSelected].Bands[fBandSelected]._10min_mean);
  sprintf(tenminl,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger10min_lo);
  sprintf(tenminh,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger10min_hi);
  
  sprintf(thirty,"%f",Channels[fChannelSelected].Bands[fBandSelected]._30min_mean);
  sprintf(thirtyl,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger30min_lo);
  sprintf(thirtyh,"%f",Channels[fChannelSelected].Bands[fBandSelected].trigger30min_hi);
  
  fLabelStats[5]->SetText(one);
  fLabelStats[6]->SetText(onetl);
  fLabelStats[7]->SetText(oneth);
  fLabelStats[9]->SetText(six);
  fLabelStats[10]->SetText(sixtl);
  fLabelStats[11]->SetText(sixth);
  fLabelStats[13]->SetText(twelve);
  fLabelStats[14]->SetText(twelvetl);
  fLabelStats[15]->SetText(twelveth);
  fLabelStats[17]->SetText(twenty);
  fLabelStats[18]->SetText(twentytl);
  fLabelStats[19]->SetText(twentyth);
  
  fLabelStats[21]->SetText(onemin);
  fLabelStats[22]->SetText(oneminl);
  fLabelStats[23]->SetText(oneminh);
  //cout<<"oneminh = "<<oneminh<<endl;
  
  fLabelStats[25]->SetText(tenmin);
  fLabelStats[26]->SetText(tenminl);
  fLabelStats[27]->SetText(tenminh);
  
  fLabelStats[29]->SetText(thirty);
  fLabelStats[30]->SetText(thirtyl);
  fLabelStats[31]->SetText(thirtyh);

  if(mDebug == mDebugAll)
    { 
      cout<<"one = "<<one<<endl;
      cout<<"onetl = "<<onetl<<endl;
      cout<<"oneth = "<<oneth<<endl;
      cout<<"six "<<six<<endl;
      cout<<"sixtl "<<sixtl<<endl;
      cout<<"sixth "<<sixth<<endl;
      cout<<"twelve "<<twelve<<endl;
      cout<<"twelvetl "<<twelvetl<<endl;
      cout<<"twelveth "<<twelveth<<endl;
      cout<<"twenty "<<twenty <<endl;
      cout<<"twentytl "<<twentytl <<endl;
      cout<<"twentyth "<<twentyth <<endl;

      cout<<"onemin "<<onemin<<endl;
      cout<<"oneminl "<<oneminl<<endl;
      cout<<"oneminh "<<oneminh<<endl;

      cout<<"tenmin "<<tenmin<<endl;
      cout<<"tenminl "<<tenminl<<endl;
      cout<<"tenminh "<<tenminh<<endl;

      cout<<"thirtymin "<<thirty<<endl;
      cout<<"thirtyminl "<<thirtyl<<endl;
      cout<<"thirtyminh "<<thirtyh<<endl;
    } 
  
  delete mean_histogram;
  
}//END GETMEANVALUES

void SpecMonWindow::Sockets(string type)
{
  if(fSocketUse == false){
    
    fSocketUse=true;
    //    cout<<"fSocketUse = true"<<endl;
    
    fLogScale->SetState(kButtonDisabled);
    fLogScaleX->SetState(kButtonDisabled);
    fLogScaleStripY->SetState(kButtonDisabled);
    fLogScaleStripX->SetState(kButtonDisabled);
    fExit->SetState(kButtonDisabled);
    fUpdate->SetState(kButtonDisabled);
    
    fBinsSelected = fBins->GetNumber();
    
    fHeartbeat->Reset();
    Bool_t kTrue= true;
    bool histogram_bool = false;
    bool stripchart_bool = false;
    bool histogram_mean = false;
    
    char str[32];
    ss = new TServerSocket(9098,kTrue);
    TSocket *socket = ss->Accept();
    
    if(type =="update")
      {
	socket->Send("update");             	//send message to backgroud monitor that the gui needs an update-------------------- Socket 1->
	socket->Recv(str,32);                   //receive the response from the backgroup monitor---------------------------------- Socket 2<-
	
	if(strcmp(str,"go") ==0)
	  {
	    if(mDebug == mDebugAll)
	      {
		cout<<"Socket received go"<<endl;
	      }
	  }else if(strcmp(str,"go") !=0){
	    cout<<"error in socket 2"<<endl;
	    string eObj = "Error in Socket 2";
	    throw runtime_error(eObj);
	  }
      
	if(strcmp(str,"go")==0)
	  {
	    TMessage message(kMESS_OBJECT);
	  
	    TH1F *info = new TH1F("info","#bin,channelnumber, bandnumber",3,0,15);
	    info->Fill(0.0,fBinsSelected);
	    info->Fill(7.0,fChannelSelected);
	    info->Fill(14.0,fBandSelected);
	    	  
	    //message.Reset();
	    message.WriteObject(info);
	  
	    socket->Send(message);          //send the # of bins, channel# and bandnumber to the bg ---------------------------- Socket 3->
	  
	    if(mDebug == mDebugAll)
	      {
		cout<<"setn the number of bins, channelnum, bandnum to bg - socket 3"<<endl;
	      }
	    
	    delete info;
	  }
	
	socket->Recv(str,32);                   //get the ready signal ------------------------------------------------------------- Socket 4<-
      
	if(strcmp(str,"ready")==0)
	  {
	    socket->Send("ready");          //send message to bg that gui is ready to receive ---------------------------------- Socket 5->
	  }else{
	    string eObj = "Error at socket 4 in gui";
	    throw runtime_error(eObj);
	  }
      }else if(type == "testbandupdate"){
	//this is the stuff for the test band 
	
	if(mDebug == mDebugAll)
	  {
	    cout<<"before sending testbandupdate"<<endl;
	  }

	if(FirstUpdateTestBand)
	  {
	    socket->Send("testbandupdatefirst");  //--------------------------------------------------Socket 1t-a->
	    if(mDebug == mDebugAll)
	      {
		cout<<"Sending testbandupdate_first"<<endl;
	      }
	  }else{
	    socket->Send("testbandupdate");        //--------------------------------------------------Socket 1t-b->
	  }
	
	socket->Recv(str,32);			       //receive confirm from BGM--------------------------Socket 2t<-
	
	if(strcmp(str,"go") ==0 && mDebug == mDebugAll)
	  {
	    cout<<"Socket received go"<<endl;
	  }else if(strcmp(str,"go") !=0){
	    cout<<"error in socket 2t"<<endl;
	    string eObj = "Error in Socket 2t";
	    throw runtime_error(eObj);
	  }
	
	if(strcmp(str,"go") == 0)
	  {
	    if(FirstUpdateTestBand)
	      {	      
		TMessage message(kMESS_OBJECT);
		
		TH1F *newband = new TH1F("newband info","freqlo, freqhi, powerhi, powerlo,channelnumber,bandnumber",6,0,30);//ths first thing is so that the BGM is able to tell if this is the first time for a test band
		newband->Fill(0.0,Channels[fChannelSelected].Bands[fBandSelected].freqlo);
		newband->Fill(7.0,Channels[fChannelSelected].Bands[fBandSelected].freqhi);
		newband->Fill(14.0,Channels[fChannelSelected].Bands[fBandSelected].powerlo);
		newband->Fill(19.0,Channels[fChannelSelected].Bands[fBandSelected].powerhi);
		newband->Fill(24.0,fChannelSelected);
		newband->Fill(29.0,fBandSelected);		
		
		message.WriteObject(newband);
		
		socket->Send(message);		         //---------------------------------------------------Socket 3t ->
		delete newband;
	      }else{//this is for after the first time that the test band is defined
		
		TMessage message(kMESS_OBJECT);
		
		TH1F *info = new TH1F("info","#bin,channelnumber, bandnumber",3,0,15);
		info->Fill(0.0,fBinsSelected);
		info->Fill(7.0,fChannelSelected);
		info->Fill(14.0,fBandSelected);
		
		message.WriteObject(info);
	      
		socket->Send(message);          //send the # of bins, channel# and bandnumber to the bg ---------------------------- Socket 3->
	      
		if(mDebug == mDebugAll)
		  {
		    cout<<"setn the number of bins, channelnum, bandnum to bg - socket 3"<<endl;
		  }
	      
		delete info;
		socket->Recv(str,32);                   //get the ready signal ------------------------------------------------------------- Socket 4<-
		
		if(strcmp(str,"ready")==0)
		  {
		    socket->Send("ready");          //send message to bg that gui is ready to receive ---------------------------------- Socket 5->
		  }else{
		    string eObj = "Error at socket 4 in gui";
		    throw runtime_error(eObj);
		  }	
	      }
	  }
      }
    
    ss->Close();	

    if(type =="testbandupdate" && FirstUpdateTestBand ==true)
      {
	//this sets it up so that the buttons are now on.
	fSocketUse=false;
	fLogScale->SetState(kButtonUp);
	fLogScaleX->SetState(kButtonUp);
	fLogScaleStripY->SetState(kButtonUp);
	fLogScaleStripX->SetState(kButtonUp);
	fExit->SetState(kButtonUp);
	fUpdate->SetState(kButtonUp);
	//don't do anything
      }else{
	mon = new TMonitor;
	mon->Add(socket);
      }
    
    if(!FirstUpdateTestBand)
      {
	while(1)
	  {
	    //cout<<"in while"<<endl;	  
	    TMessage *mess;
	    TSocket *s;
	    s= mon->Select();
	    s->Recv(mess);           //get the message from the bg --------------------------------------------------------------- Socket 6-??<-
	    
	    if(mess->What() == kMESS_STRING)
	      {
		char str[64];
		mess->ReadString(str,64);
		
		if(strcmp(str,"Finished") ==0) //------------------------------------------------------------------------------ Socket 15<-
		  {
		  if(mDebug == mDebugAll)
		    {
		      printf("client%s/n",str);
		    }
		    mon->Remove(s);
		    fSocketUse=false;
		    fLogScale->SetState(kButtonUp);
		    fLogScaleX->SetState(kButtonUp);
		    fLogScaleStripY->SetState(kButtonUp);
		    fLogScaleStripX->SetState(kButtonUp);
		    fExit->SetState(kButtonUp);
		    fUpdate->SetState(kButtonUp);

		  }else if(strcmp(str,"histogram")==0){ //--------------------------------------------------------------------- Socket 9<-
		    histogram_bool =true;
		    socket->Send("ready");            //send confirm--------------------------------------------------------- Socket 10->
		  }else if(strcmp(str,"stripchart")==0){ //------------------------------------------------------------------- Socket 6<-
		    stripchart_bool =true;
		    gStyle->SetHistLineColor(0);
		    socket->Send("ready");             //send confirm ---------------------------------------------------- Socket 7->
		  }else if(strcmp(str,"mean_values") ==0){//----------------------------------------------------------------Socket 12<-
		    histogram_mean =true;
		    socket->Send("ready");//send confirm -----------------------------------------------------------------Socket 13->
		  }else{
		    string eObj = "error in socket while loop";
		    throw runtime_error(eObj);
		  }
	      }else if(mess->What() == kMESS_OBJECT){
		
		
		if((type =="update" || type =="testbandupdate") && stripchart_bool ==true)//-------------------------------------- Socket 8<-
		  {
		    GetStripChart(mess);
		    stripchart_bool = false;
		  }else if((type =="update" || type =="testbandupdate")&& histogram_bool ==true){ //---------------------------- Socket 11<-
		    
		    GetHistogram(mess);
		    histogram_bool = false;
		  }else if((type =="update" || type =="testbandupdate") && histogram_mean ==true){ //--------------------------- Socket 14<-
		    GetMeanValues(mess);
		    histogram_mean = false;
		  }
	      }
	    delete mess;
	  }

	socket->Close();	
      }

    fSocketUse=false;
    fLogScale->SetState(kButtonUp);
    fLogScaleX->SetState(kButtonUp);
    fLogScaleStripY->SetState(kButtonUp);
    fLogScaleStripX->SetState(kButtonUp);
    fExit->SetState(kButtonUp);
    fUpdate->SetState(kButtonUp);
    
    FirstUpdateTestBand = false;    
    
  }else if(fSocketUse){
    cout<<"Wait until the socket has sent all of its information"<<endl;
  }
  
  cout<<"the end of the socket"<<endl;
  return;

}//END SOCKET

//-----------------------------------------------------------------------------------
void SpecMonWindow::BandSelectUpdate(int channelnumber, int numbands)
{
  if(numbands == kBandSelectIdiot)
    {
      fBandSelect->RemoveEntry(numbands);
    }else{
      for(int i=0;i<=numbands;++i)
	{
	  fBandSelect->RemoveEntry(i);
	}
      fBandSelect->RemoveEntry(kTestBand);
    }
  
  for(int i =0;i<Channels[channelnumber].BandNum;++i)
    {
      Int_t temp = i;
      string entry = Channels[channelnumber].Bands[i].BandName;
      
      fBandSelect->AddEntry(entry.c_str(),temp);
    }
  fBandSelect->AddEntry("Test Band",kTestBand);
  
  NumberBandsToDelete = Channels[channelnumber].BandNum;
  
  return;
}//END BANDSELECTUPDATE

void SpecMonWindow::UpdateStripChart(vector<float> power)
{
  string temp = Channels[fChannelSelected].Bands[fBandSelected].BandName;
  temp = temp + " " + " 6 Hr Strip Chart";
  
  stripchart = new TH1F("stripchart", temp.c_str(),21600,0,21599);
  
  for(int i =0;i<21600;++i)
    {
      stripchart->Fill(i,power[i]);
    }
  
  fStripChart->cd();
  stripchart->Draw();
  fStripChart->Update();
  stripchart->GetXaxis()->SetTitle("Time (seconds)");
  stripchart->GetYaxis()->SetTitle("Integrated Power (ADC^2)");
  stripchart->GetXaxis()->CenterTitle();
  stripchart->GetYaxis()->CenterTitle();
  stripchart->SetTitle("6 hr Strip Chart");
  //stripchart->SetLabelOffset(1.2);

  //stripchart->GetXaxis()->Set
  fStripChart->Modified();
  
  return;	
}//END UPDATESTRIPCHART

//______________________________________________________________________________
Bool_t SpecMonWindow::ProcessMessage (Long_t msg, Long_t parm1, 
				      Long_t parm2)
{
  if((GET_MSG (msg) == kC_COMMAND) && (GET_SUBMSG (msg) == kCM_COMBOBOX))
    {
      if(parm1 == kChannelSelect)
	{
	  cout<<"you have selected channel number "<<parm2<<endl;
	  fChannelSelected = parm2;
	  if(parm2 != 1000)
	    {
	      BandSelectUpdate(parm2,NumberBandsToDelete);
	    }}
      
      else if(parm1 == kBandSelect)
	{
	  if(parm2 != kBandSelectIdiot)
	    {
	      if(parm2 != kTestBand)
		{
		  fBandSelected = parm2;
		  //change the values in the number entry boxes of freq and power
		  fFreq[0]->SetNumber(Channels[fChannelSelected].Bands[fBandSelected].freqlo);
		  fFreq[1]->SetNumber(Channels[fChannelSelected].Bands[fBandSelected].freqhi);
		  fPower[0]->SetNumber(Channels[fChannelSelected].Bands[fBandSelected].powerlo);
		  fPower[1]->SetNumber(Channels[fChannelSelected].Bands[fBandSelected].powerhi);
	      
		  if(BandTestingMode)
		    {
		      BandTestingMode = false;
		      Channels[fChannelSelected].Bands.pop_back();
		      Channels[fChannelSelected].BandNum = (Channels[fChannelSelected].BandNum -1);
		    }
		  
		  cout<<"You have selected band number "<<parm2<<endl;
		  Sockets("update");
		  
		}else if(parm2 == kTestBand){//this give you a blank slate to define a band in the selected channel.  If you want a different channel that is not monitored  add it.		  
		  float freql = fFreq[0]->GetNumber();
		  float freqh = fFreq[1]->GetNumber();
		  float powerl = fPower[0]->GetNumber();
		  float powerh =fPower[1]->GetNumber();
 
		  fFreq[0]->SetNumber(freql);
		  fFreq[1]->SetNumber(freqh);
		  fPower[0]->SetNumber(powerl);
		  fPower[1]->SetNumber(powerh);

		  //the for loop is not really needed
		  
		  int bandnum = Channels[fChannelSelected].BandNum;
		  
		  if(Channels[fChannelSelected].Bands[(bandnum-1)].BandName == "Test Band")
		    {
		      Channels[fChannelSelected].Bands.pop_back();
		      Channels[fChannelSelected].BandNum = (Channels[fChannelSelected].BandNum -1);
		    }
  
		  BandTestingMode = true;
		  FirstUpdateTestBand = true;
		  
		  BandInfo temp;
		  temp.freqlo = freql;
		  temp.freqhi =freqh;
		  temp.powerlo = powerl;
		  temp.powerhi = powerh;

		  ++Channels[fChannelSelected].BandNum;
		  Channels[fChannelSelected].Bands.insert(Channels[fChannelSelected].Bands.end(),temp);
		  fBandSelected = (Channels[fChannelSelected].BandNum -1);
		  Channels[fChannelSelected].Bands[fBandSelected].BandName = "Test Band";
		  
		  cout<<"You must set your limits and then update to start the trial"<<endl;
		}
		  
	    }else if(parm2 == kBandSelectIdiot){
	      //selected a band without first selecting a channel
	      cout<<"Select a band also"<<endl;
	    }
	}
    }  
  
  // Buttons
  else if ((GET_MSG (msg) == kC_COMMAND) &&
	   (GET_SUBMSG (msg) == kCM_BUTTON)) {
    
    if(parm1 == kButtonLogScale)
      {
	fHistogram->cd();
	if(logscale == false)
	  {
	    gPad->SetLogy(0);
	    logscale = true;
	  }else if(logscale){
	    
	    gPad->SetLogy(1);
	    logscale = false;
	  }
	    fHistogram->Modified();
	    fHistogram->Update();
      }else if(parm1 == kButtonExit){
	
	CloseWindow();

      }else if(parm1 == kButtonUpdate){

	if(BandTestingMode)
	  {
	    Sockets("testbandupdate");
	  }else{
	    Sockets("update");
	  }

      }else if(parm1 == kButtonLogScaleX){

	fHistogram->cd();
	if(logscalex == false)
	  {
	    gPad->SetLogx(0);
	    logscalex = true;
	  }else if(logscalex){
	    gPad->SetLogx(1);
	    logscalex = false;
	  }
	
	fHistogram->Modified();
	fHistogram->Update();
      }else if(parm1 == kButtonLogScaleStrip){

	fStripChart->cd();
	if(logscalestrip == false)
	  {
	    //stripchart->SetLogy(0);
	    gPad->SetLogy(0);
	    logscalestrip = true;
	  }else if(logscalestrip){
	    gPad->SetLogy(1);
	    logscalestrip = false;
	  }
	
	fStripChart->Modified();
	fStripChart->Update();
      }else if(parm1 == kButtonLogScaleStripX){

	fStripChart->cd();
	if(logscalestripx == false)
	  {
	    gPad->SetLogx(0);
	    logscalestripx = true;
	  }else if(logscalestripx){
	    gPad->SetLogx(1);
	    logscalestripx = false;
	  }
	
	fStripChart->Modified();
	fStripChart->Update();
      }
    
    switch (parm1) {
      // Pause
    case kButtonReset:
      {
	ReadConf();
	break;
      }
      // Pause
    case kButtonPause:
      {
	fConf->fPause = true;
	fButton[0]->SetState (kButtonDisabled);
	fButton[1]->SetState (kButtonUp);
	
	break;
      }
      // Resume
    case kButtonResume:
      {
	Sockets("update");
	fConf->fPause = false;
	fButton[0]->SetState (kButtonUp);
	fButton[1]->SetState (kButtonDisabled);
	break;
      }
    }
  }
  // set configuration record if a new channel is selected
  else if ((GET_MSG (msg) == kC_COMMAND) && 
	   (GET_SUBMSG (msg) == kCM_COMBOBOX)) {
    switch (parm1) {
    case kChnSel:
      ReadConf();
      break;
    }
  }
  
  else if((GET_MSG(msg)) == kC_TEXTENTRY && (GET_SUBMSG(msg)) == kTE_TEXTUPDATED){
    
    if(parm1 == kButtonBins)
      {
	fBinsSelected = fBins->GetNumber();
	cout<<"bin # is "<<fBinsSelected<<endl;
      }else if(parm1 == kFreqLo){
	
	if(BandTestingMode)
	  {
	    newfreqlo = fFreq[0]->GetNumber();
	    cout<<"lo frequency for new band is "<<newfreqlo<<endl;
	    FirstUpdateTestBand = true;	   
	    Sockets("testbandupdate");
	  }	
	
      }else if(parm1 == kFreqHi){
	
	if(BandTestingMode)
	  {
	    newfreqhi = fFreq[1]->GetNumber();
	    cout<<" Hi frequency for new band is "<<newfreqhi<<endl;	   
	    FirstUpdateTestBand = true;	    
	    Sockets("testbandupdate");
	  }
	//change freq hi 
      }else if(parm1 == kPowerLo){
	
	if(BandTestingMode)
	  {
	    newpowerlo = fPower[0]->GetNumber();
	    cout<<"Lo power for new band is "<<newpowerlo<<endl;
	    FirstUpdateTestBand = true;	   
	    Sockets("testbandupdate");
	  }
	
	//change power lo slider
      }else if(parm1 == kPowerHi){
	
	if(BandTestingMode)
	  {
	    newpowerhi = fPower[1]->GetNumber();
	    cout<<"Lo power for new band is "<<newpowerhi<<endl;
	    FirstUpdateTestBand = true;
	    Sockets("testbandupdate");	
	  }
	//change power hi slider
      }  
  }
  return kTRUE;
}
//______________________________________________________________________________
Bool_t SpecMonWindow::HandleTimer (TTimer* timer)
{
  // sometimes skip a heartbeat to avoid overloading the GUI
  if (fSkipHeartbeats) {
    fSkipHeartbeats--;
    timer->Reset();
    return kTRUE;
  }
  if(BandTestingMode)
    {
      Sockets("testbandupdate");
    }else{
      Sockets("update");
    }

  // reset timer and return
  fSkipHeartbeats = 3;
  timer->Reset();
  return kTRUE;
}

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// GUI thread                                                           //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
extern "C" {
  static void* guithread (void*)
  {
    gApplication->Run();
    return 0;
  }
}

//////////////////////////////////////////////////////////////////////////
//                                                                      //
//           SpecMon                                                    //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
SpecMon::SpecMon (int argc, const char* argv[])
  : DatEnv (argc, argv),
    mDebug(0),      
    mName("SpecMon"),
    mTrigType("Power Variations"),
    mTestBand(false),
    MaxFrame(999999),
    mFrameCount(0),
    mChannelNumber(0),
    dtotalbands(0),
    mFork(false),
    mAccuracy(1),
    guibins(20),
    mOsclist(mIn, 0),
    mTotalRuntime(0),
    fMainWindow (0),
    fFirstTime (true),
    fTS (0)
    
{
  //  cout<<"Hello World"<<endl;
  //getDacc().addChannel("H1:LSC-AS_Q");   	
  Time now;
  
  bool ifoset_p = false;

  for (int i = 0; i < argc; ++i) {
    if (strcmp(argv[i], "-debug") == 0) {
      ++i;
      mDebug = atoi(argv[i]);
    }
    
    if (strcmp(argv[i], "-ifo") == 0) {
      ++i;
      
      if (i > argc-1 || argv[i] == NULL) {
	std::string eObj = "Must specify IFO name (H1, H2, or L1)";
	cerr << eObj << endl;
	throw runtime_error(eObj);
      }

      mIFOname = argv[i];
      ifoset_p = true;
            
      if (mIFOname != "H1" && mIFOname != "H2" && mIFOname != "L1") {
	std::string eObj = "Must specify IFO name (H1, H2, or L1)";
	cerr << "Must specify IFO name (H1, H2, or L1)" << endl;
	throw runtime_error(eObj);
      }
    }
      
    if (strcmp(argv[i], "-debugtype") == 0) {
      ++i;
      string debugtype = argv[i];
      if(debugtype == "parseval")
	{
	  mDebug = mDebugParseval;
	}else if(debugtype == "configfile"){
	  mDebug = mDebugConfigfile;
	}else if(debugtype == "problemactions"){
	  mDebug = mDebugProblemActions;
	}else if(debugtype == "all"){
	  mDebug = mDebugAll;
	}else if(debugtype == "alarms"){
	  mDebug = mDebugAlarms;
	}else if(debugtype == "triggers"){
	  mDebug = mDebugTriggers;
	}else if(debugtype == "test"){
	  mDebug = mDebugTest;
	}	      
    }   
  }
  //read in first part of config file
  
  string mConfigName = mName + ".conf";
  cout<<mConfigName<<endl;
  IFO = mIFOname;	
	
  vector<string> info = readconfig(mConfigName,mIFOname);
  int infoposition = 0;
  
//loop for debug purposes to see what comes out of the config file
  if(mDebug == mDebugConfigfile || mDebug == mDebugAll)
    {
        for(size_t i=0;i<info.size();++i)
	  {
	    cout<<info[infoposition]<<endl;
	    ++infoposition;
	  }
    }	
	
  infoposition =0;//reset what position you will iterate at so that the config info is put into Channel and BandInfo structures
  
  mChannelNumber = atoi(info[0].c_str());
  ++infoposition;
  
  if(mDebug == mDebugConfigfile || mDebug == mDebugAll)
  {
  cout<<"mChannelNumber = "<<mChannelNumber<<endl;
  cout<<"Time = "<<now.getS()<<endl;
  }
  
  //_______________________ CHANNEL LOOP__________________________
	
  for(int i=0;i<mChannelNumber;++i)//channel loop
    {
      ChannelInfo Channel_temp;
      
      Channel_temp.ChannelName = info[infoposition];
      
      if(mDebug == mDebugConfigfile || mDebug == mDebugAll)
	{
	  cout<<"Channel Name is "<<Channel_temp.ChannelName<<endl;
	}      
      ++infoposition;
      getDacc().addChannel(Channel_temp.ChannelName.c_str());
      
      Channel_temp.BandNum = atoi(info[infoposition].c_str());
      ++infoposition;
      
      
      for(int a=0;a<Channel_temp.BandNum;++a)//band loop
	{
	  BandInfo Band;
	  
	  Band.BandName = info[infoposition];
	  ++infoposition;
	  
	  Band.freqlo = atof(info[infoposition].c_str());
	  ++infoposition;
	  
	  Band.freqhi = atof(info[infoposition].c_str());
	  ++infoposition;
			
	  Band.powerlo = atof(info[infoposition].c_str());
	  ++infoposition;
			
	  Band.powerhi = atof(info[infoposition].c_str());
	  ++infoposition;
			
	  Channel_temp.Bands.insert(Channel_temp.Bands.end(),Band);	
	}
				
      Channels.insert(Channels.end(),Channel_temp);
      Channel_temp.Bands.clear();
	
    }
    
  if(mDebug == mDebugConfigfile || mDebug == mDebugAll)
    {
      for(int i=0;i<mChannelNumber;++i)
	{
	  cout<<"Channel "<<i<<" name is " << Channels[i].ChannelName<<endl;
	  cout<<"# of Band in Channel "<<i<<" is "<<Channels[i].BandNum<<endl;
	  int BandNum_temp= Channels[i].BandNum;
	  vector<BandInfo> tim;
	  tim = Channels[i].Bands;
	  	  
	  for(int a=0;a<BandNum_temp;++a)
	    {
	      cout<<"Band " <<a<<" name is "<<tim[a].BandName<<endl;
	      cout<<"Freqlo is "<<tim[a].freqlo<<endl;
	      cout<<"Freqhi is "<<tim[a].freqhi<<endl;
	      cout<<"Powerhi is "<<tim[a].powerhi<<endl;
	      cout<<"Powerlo is "<<tim[a].powerlo<<endl;
	      ++dtotalbands;
	    }
	  tim.clear();
	}
    }
         
  //__________________________________________________________Logfile Variable so that it is possible to output data from each band to separate files_____________
  //
  //I will eventually have this so that you can have one channel output to a file or have both processes happen at the same time.  

  for(int i=0;i<mChannelNumber;++i)
    {  
      vector<BandInfo> mBand_temp = Channels[i].Bands;
      
      for(int a =0;a<Channels[i].BandNum;++a)
	{
	  string ChannelName = Channels[i].ChannelName;
	  string BandName = mBand_temp[a].BandName;
	  string name_temp= ChannelName +"_"+ BandName +".txt";
	  name_temp = "logs/"+name_temp;
	  dLogfilename.insert(dLogfilename.end(),name_temp);
	  //cout<<"logfile names = "<<name_temp<<endl;
	} 
    }
  //____________________________________________________________________________________________________________________

	
  // Write to $DMTHTMLOUT (set by ProcessManager) or
  // $GDSAPACHE/monitor_reports/mName 
  char *htmldir = getenv("DMTHTMLOUT");
  if (!htmldir) {
    cerr << mName << ": warning -- environment variable DMTHTMLOUT "
	 << "not set. Trying GDSAPACHE." << endl;
    
    htmldir = getenv("GDSAPACHE");
    if (!htmldir) {
      cerr << mName << ": warning -- environment variable GDSAPACHE "
	   << "not set. Writing log and duty cycle files to "
	   << "current directory" << endl;
    } else {
      cerr << mName << ": Writing log and duty cycle files to "
	   << "$GDSAPACHE/monitor_reports/" << mName << "/"
	   << endl;
      
      mHtmlDir  = htmldir;
      mHtmlDir += "/monitor_reports/" + mName + "/";
    }
  } else {
    cerr << mName << ": Writing log and duty cycle files to "
	 << htmldir << endl;
    
    mHtmlDir = htmldir;
    mHtmlDir += "/";
  }
  
  mLogfileName   = "SpecMon_Log.txt";
  mIndexPageName = mHtmlDir + "index.html";
  mDispfileName  = mHtmlDir + "SpecMon_Status.html";
  
  std::string urldir;
  if (mIFOname == "H1")
    urldir = "http://blue.ligo-wa.caltech.edu/gds/Alarms/" + mName;
  else
    urldir = "http://london.ligo-la.caltech.edu/gds/Alarms/" + mName;
          
  SetAlarmDesc(urldir,Channels,mChannelNumber);
       
  if(mDebug==mDebugAlarms || mDebug == mDebugAll)
    {
      for(int i=0;i<mChannelNumber;++i)
	{
	  for(int a=0;a<Channels[i].BandNum;++a)
	    {
	      cout<<"AlarmDesc = "<<Channels[i].Bands[a].AlarmDesc<<endl;
	    }
	}
    }
    
  if (mDebug > 0 || mDebug == mDebugAll) {
    std::cerr << mName << ": mLogfileName = " << mLogfileName << std::endl;
    std::cerr << mName << ": mIndexPageName = " << mIndexPageName << std::endl;
    std::cerr << mName << ": mDispfileName = " << mDispfileName << std::endl;
  }

////////////////////////////////
    ///writeIndexPage();
////////////////////////////////

  // Open log with append mode
  mLogfile.open(mLogfileName.c_str(), ios::app);
  if(mDebug == mDebugAll)
    cout<<"after logfile open"<<endl;  

  if (!mLogfile) {
    cerr << mName << ": could not open logfile " << mLogfileName << endl;
    cerr << "          reverting to stdout" << endl;
    mLog = &std::cout;
  } else {
    mLog = &mLogfile;
  }
        
    //--------------------------Set Alarm Stuff--------------------------------------------
    
  // Set alarm client name
  mAlarmClient.setName(mName.c_str());
  cout<<"after alarm client set"<<endl;
       
  //possibly have this set in the config file not sure.  also have to eventually change it so that there is alarms for both
  //lo and hi problems along with different severitys or alarms for bigger ranges
  
  // Set alarm data and handles
  Interval timeout(10.0);
  int      severity(1);  // it is not specified in the AlarmData
                         // documentation how severities are ordered
                                            
  for(int i=0;i<mChannelNumber;++i)
    {
      for(int a=0;a<Channels[i].BandNum;++a)
	{
	  string name = Channels[i].ChannelName + "_" + Channels[i].Bands[a].BandName;
	  AlarmData temp(mName,name,timeout,severity,Channels[i].Bands[a].AlarmDesc);
	  Channels[i].Bands[a].AD = temp;
	    
	  //return code for alarm settings
	  lmsg::error_t rc;
	  rc = mAlarmClient.defineAlarm(Channels[i].Bands[a].AD);
	  
	  if(rc!= 0)
	    {
	      //failure
	      *mLog << mName<<": error defining alarm: "<<Channels[i].Bands[a].BandName<<" problem: return code "<<rc<<endl;
	    }
	}
    }     
}

//______________________________________________________________________________
SpecMon::~SpecMon()
{
  // get out 
  delete fTS;
  getDacc().close();
  gApplication->Terminate(0);
}//END ~SPECMON

void SpecMon::SetAlarmDesc(string urldir,vector<ChannelInfo> &Channels,int ChannelNumber)
{
  for(int i=0;i<ChannelNumber;++i)
    {
      string channelname = Channels[i].ChannelName;
      
      for(int a=0;a<Channels[i].BandNum;++a)
	{
	  string bandname = Channels[i].Bands[a].BandName;
	  Channels[i].Bands[a].AlarmDesc = urldir + "_" + channelname + "_" + bandname + "_alarmDesc.html";
	}
    }
}//END SETALARMDESC 
   
void SpecMon::BandProblemActions(vector<ChannelInfo> &bChannels,int bChannelNum,int bBandNum,Time timeNow,bool testband,vector<int> &btriggers,vector<size_t> &btrigger_times,vector<Time> &btrigger_times_t)
{
  //  int triggerplace = mFrameCount%(86400/15);
  //need to add slash starhere
  
  if(bChannels[bChannelNum].Bands[bBandNum].hi_status == true)
    {
      ++bChannels[bChannelNum].Bands[bBandNum].BandErrorCount_hi;
      //bChannels[bChannelNum].Bands[bBandNum].LastProblemTime_hi = timeNow;
      //bChannels[bChannelNum].Bands[bBandNum].LastProblemFrame_hi = mFrameCount;
      bChannels[bChannelNum].Bands[bBandNum].trigger_times.insert(bChannels[bChannelNum].Bands[bBandNum].trigger_times.end(),mFrameCount);
      Channels[bChannelNum].Bands[bBandNum].trigger_times.insert(Channels[bChannelNum].Bands[bBandNum].trigger_times.end(),mFrameCount);
      Channels[bChannelNum].Bands[bBandNum].triggers.insert(Channels[bChannelNum].Bands[bBandNum].triggers.end(),3);
      bChannels[bChannelNum].Bands[bBandNum].triggers.insert(bChannels[bChannelNum].Bands[bBandNum].triggers.end(),3);
      //btriggers.insert(btriggers.end(),3);//3 is for exceeds hi power
      //btrigger_times.insert(btrigger_times.end(),mFrameCount);
      //btrigger_times_t.insert(btrigger_times_t.end(),timeNow);
    }else{
      
      ++bChannels[bChannelNum].Bands[bBandNum].BandErrorCount_lo;
      bChannels[bChannelNum].Bands[bBandNum].LastProblemTime_lo = timeNow;
      bChannels[bChannelNum].Bands[bBandNum].LastProblemFrame_lo = mFrameCount;

      bChannels[bChannelNum].Bands[bBandNum].trigger_times.insert(bChannels[bChannelNum].Bands[bBandNum].trigger_times.end(),mFrameCount);
      bChannels[bChannelNum].Bands[bBandNum].triggers.insert(bChannels[bChannelNum].Bands[bBandNum].triggers.end(),5);

      Channels[bChannelNum].Bands[bBandNum].trigger_times.insert(Channels[bChannelNum].Bands[bBandNum].trigger_times.end(),mFrameCount);
      Channels[bChannelNum].Bands[bBandNum].triggers.insert(Channels[bChannelNum].Bands[bBandNum].triggers.end(),5);

      //btriggers.insert(btriggers.end(),5);//5 is for less than low power
      //btrigger_times.insert(btrigger_times.end(),mFrameCount);
      //btrigger_times_t.insert(btrigger_times_t.end(),timeNow);
    }

  if(bChannels[bChannelNum].Bands[bBandNum].hi_status == true)
    {
      *mLog << mName<< ":Entered  BandProblemActions high for "<<bChannels[bChannelNum].Bands[bBandNum].BandName<< "  at "<< timeNow <<endl;
    }else{
      *mLog << mName<< ":Entered  BandProblemActions low for "<<bChannels[bChannelNum].Bands[bBandNum].BandName<< "  at "<< timeNow <<endl;
    }
  
  //if(testband == false)
  //{
  string alarm_trig;//in place of trig subtype and alarm name
      
  alarm_trig = bChannels[bChannelNum].ChannelName +"_"+ bChannels[bChannelNum].Bands[bBandNum].BandName;
  
  if(mDebug == mDebugProblemActions || mDebug == mDebugAll)
    {
      cout<<alarm_trig<<endl; 
    }
  
  //set up triggers 		
  trig::TrigRslt cricket(mTrigType.c_str(),alarm_trig.c_str(),0);
  cricket.setIfos(mIFOname.c_str());
  cricket.setTime(timeNow);
  cricket.setPriority(trig::p_warn);//this will eventually need to be changed for how much the power is above
  
  //send trigger to metaDB
  cricket.setDisposition(trig::d_metaDB +trig::d_alarm);
  
  trig::TrigBase::index_type rc = sendTrigger(cricket);
  
  if (rc)
    {
      *mLog << mName << ": Error sending trigger: error code "
	    << rc << endl;
    }else{
      *mLog << mName << ": Trigger successfully sent" << endl;
    }    	
  
  
  lmsg::error_t rc2 = mAlarmClient.setAlarm(bChannels[bChannelNum].Bands[bBandNum].AD, bChannels[bChannelNum].Bands[bBandNum].AH);
  if(rc2)
    {
      *mLog<<mName<<": Error setting alarm for "<<bChannels[bChannelNum].Bands[bBandNum].BandName<<": error code "<<rc2<<endl;
    }else{
      *mLog<<mName<<": Alarm "<<bChannels[bChannelNum].Bands[bBandNum].BandName<<" set successfully"<<endl;
    }
  // }
  
}//END BANDPROBLEMACTIONS  

//-----------------------------------------------------------------------------------------------

void SpecMon::SetBools(bool &one,bool &ten, bool &thirty, bool &hour, bool &sixh, bool &twelveh, bool &day, int framecount)
{
  if(framecount >60)
    {
      one = true;
    }else{
      one = false;
    }
  
  if(framecount >600)
    {
      ten =true;
    }else{
      ten =false;
    }
  
  if(framecount >1800)
    {
      thirty =true;
    }else{
      thirty =false;
    }
  
  if(framecount >3600)
    {
      hour =true;
    }else{
      hour =false;
    }
  
  if(framecount >21600)
    {
      sixh =true;
    }else{
      sixh =false;
    }
  
  if(framecount >43200)
    {
      twelveh =true;
    }else{
      twelveh =false;
    }
  
  if(framecount >86400)
    {
      day =true;
    }else{
      day =false;
    }
  
}//END SETBOOLS

vector<int> SpecMon::GetTriggerInfo(int channelnumber, int bandnumber, int mFramecount)
{
  bool onemin;
  bool tenmin;
  bool thirtymin;
  bool hour;
  bool sixh;
  bool twelveh;
  bool day;//these are true if the slot has to be filled, eg onemin is true if SpecMon has been running at least 60 secs and has not been filled
  vector<int> output(7,0);
  SetBools(onemin, tenmin,thirtymin,hour,sixh,twelveh, day,mFramecount);
  if(Channels[channelnumber].Bands[bandnumber].trigger_times.size() != 0)
  {
    for(size_t i=0;i<Channels[channelnumber].Bands[bandnumber].trigger_times.size();++i)//not sure about the triggertimes stuff
      {
	size_t temp = Channels[channelnumber].Bands[bandnumber].trigger_times.size() - i;
	
	if(mFrameCount-Channels[channelnumber].Bands[bandnumber].trigger_times[temp]>60)
	  {
	    if(onemin)
	      {
		output[0]=i;
		onemin = false;
	      }
	  }
	
	if(mFrameCount-Channels[channelnumber].Bands[bandnumber].trigger_times[temp]>600)
	  {
	    if(tenmin)
	      {
		output[1]=i;
		tenmin = false;
	      }
	  }
	
	if(mFrameCount-Channels[channelnumber].Bands[bandnumber].trigger_times[temp]>1800)
	  {
	    if(thirtymin)
	      {
		output[2]=i;
		thirtymin = false;
	      }
	  }
	
	if(mFrameCount-Channels[channelnumber].Bands[bandnumber].trigger_times[temp]>3600)
	  {
	    if(hour)
	      {
		output[3]=i;
		hour = false;
	      }
	  }
	
	if(mFrameCount-Channels[channelnumber].Bands[bandnumber].trigger_times[temp]>3600*6)
	  {
	    if(sixh)
	      {
		output[4]=i;
		sixh = false;
	      }
	  }
	
	if(mFrameCount-Channels[channelnumber].Bands[bandnumber].trigger_times[temp]>3600*12)
	  {
	    if(twelveh)
	      {
		output[5]=i;
		twelveh = false;
	      }
	  }
	
	if((mFrameCount-Channels[channelnumber].Bands[bandnumber].trigger_times[temp])>3600)
	  {
	    if(day)
	      {
		output[6]=i;
		day = false;
	      }
	  }
      }
  }
     
  return output;
     
}//END GETTRIGGERINFO
  

void SpecMon::FillMeanHistogram(TH1S* mean, size_t framecount,int bandnumber,int channelnumber)
{
  //cout<<"in fill mean histo"<<endl;
  vector<int> triggers = GetTriggerInfo(channelnumber,bandnumber,framecount);
  Channels[channelnumber].Bands[bandnumber].trigger1_hi = triggers[0];
  //Channels[channelnumber].Bands[bandnumber].trigger6_

  mean->Fill(.3,Channels[channelnumber].Bands[bandnumber]._1hr_mean);
  mean->Fill(1.3,Channels[channelnumber].Bands[bandnumber]._6hr_mean);
  mean->Fill(2.3,Channels[channelnumber].Bands[bandnumber]._12hr_mean);
  mean->Fill(3.3,Channels[channelnumber].Bands[bandnumber]._24hr_mean);
  mean->Fill(4.3,Channels[channelnumber].Bands[bandnumber].trigger1_lo);
  mean->Fill(5.3,Channels[channelnumber].Bands[bandnumber].trigger6_lo);
  mean->Fill(6.3,Channels[channelnumber].Bands[bandnumber].trigger12_lo);
  mean->Fill(7.3,Channels[channelnumber].Bands[bandnumber].trigger24_lo);
  mean->Fill(8.3,Channels[channelnumber].Bands[bandnumber].trigger1_hi);
  mean->Fill(9.3,Channels[channelnumber].Bands[bandnumber].trigger6_hi);
  mean->Fill(10.3,Channels[channelnumber].Bands[bandnumber].trigger12_hi);
  mean->Fill(11.3,Channels[channelnumber].Bands[bandnumber].trigger24_hi);
  
  mean->Fill(12.3,Channels[channelnumber].Bands[bandnumber].trigger30min_hi);
  mean->Fill(13.3,Channels[channelnumber].Bands[bandnumber].trigger30min_lo);
  mean->Fill(14.3,Channels[channelnumber].Bands[bandnumber]._30min_mean);
  mean->Fill(15.3,Channels[channelnumber].Bands[bandnumber].trigger10min_hi);
  mean->Fill(16.3,Channels[channelnumber].Bands[bandnumber].trigger10min_lo);
  mean->Fill(17.3,Channels[channelnumber].Bands[bandnumber]._10min_mean);
  mean->Fill(18.3,Channels[channelnumber].Bands[bandnumber].trigger1min_hi);
  mean->Fill(19.3,Channels[channelnumber].Bands[bandnumber].trigger1min_lo);
  mean->Fill(20.3,Channels[channelnumber].Bands[bandnumber]._1min_mean);
}

TH1F* SpecMon::CreateStripChart(size_t framecount, int channelnumber, int bandnumber,string name,double &min, double &max)
{
  if(mDebug == mDebugAll)
    {
      cout<< "in Create stripchart"<<endl;
    }

  TH1F *stripchart;
  if(framecount<1000)
    {
      stripchart = new TH1F("stripchart",name.c_str(),1000,0,999);
    }else if(framecount>=1000){
      int min = (framecount -1000);
      int max = framecount;
      stripchart = new TH1F("stripchart",name.c_str(),1000,0,999);
    }
  
  if(framecount<1000)
    {
      min = Channels[channelnumber].Bands[bandnumber].mean_vec[0];
      max = Channels[channelnumber].Bands[bandnumber].mean_vec[0];

      for(size_t i=0;i<Channels[channelnumber].Bands[bandnumber].mean_vec.size();++i)
	{
	  stripchart->SetBinContent((i),Channels[channelnumber].Bands[bandnumber].mean_vec[i]);
	  //cout<<setw(20)<<i<<setw(20)<<Channels[channelnumber].Bands[bandnumber].mean_vec[i]<<endl;//this should be taken out soon.
	  
	  if(Channels[channelnumber].Bands[bandnumber].mean_vec[i]<min)                      //find the min and max
	    {
	      min = Channels[channelnumber].Bands[bandnumber].mean_vec[i];
	    }else if(Channels[channelnumber].Bands[bandnumber].mean_vec[i]>max){
	      max = Channels[channelnumber].Bands[bandnumber].mean_vec[i];
	    }
	}
    }else if(framecount>=1000){
      size_t start = framecount -1000;
      
      min = Channels[channelnumber].Bands[bandnumber].mean_vec[start];
      max = Channels[channelnumber].Bands[bandnumber].mean_vec[start];      

      for(size_t i = start;i<Channels[channelnumber].Bands[bandnumber].mean_vec.size();++i)
	{
	  stripchart->SetBinContent((i-start),Channels[channelnumber].Bands[bandnumber].mean_vec[i]);
	  /*
	  if((framecount -Channels[channelnumber].Bands[bandnumber].maxPowerFrame)>1000)
	    {
	      //find the min and max of distribution
	      if(i==0)
		{
		  max = Channels[channelnumber].Bands[bandnumber].mean_vec[i];
		  min = Channels[channelnumber].Bands[bandnumber].mean_vec[i];
		}else{
		  if(Channels[channelnumber].Bands[bandnumber].mean_vec[i]>max) max = Channels[channelnumber].Bands[bandnumber].mean_vec[i];
		  if(Channels[channelnumber].Bands[bandnumber].mean_vec[i]<min) min = Channels[channelnumber].Bands[bandnumber].mean_vec[i];
		}
		}*/

	  if(Channels[channelnumber].Bands[bandnumber].mean_vec[i]<min)                      //find the min and max
	    {
	      min = Channels[channelnumber].Bands[bandnumber].mean_vec[i];
	    }else if(Channels[channelnumber].Bands[bandnumber].mean_vec[i]>max){
	      max = Channels[channelnumber].Bands[bandnumber].mean_vec[i];
	    }
	}			
    }//end else if   
  if(mDebug == mDebugAll){  cout<<"end of createstripchart"<<endl; }

  stripchart->GetYaxis()->CenterTitle();
  stripchart->GetYaxis()->SetTitleOffset(1.7);
  stripchart->GetYaxis()->SetTitleSize(0.03);
  stripchart->GetXaxis()->CenterTitle();
  stripchart->GetYaxis()->SetLabelOffset(0);
  stripchart->GetYaxis()->SetLabelSize(0.03);
  //stripchart->GetXaxis()->SetExponent();


  return stripchart;
}//end CreateStripChart

TH1F* SpecMon::CreateHistogram(size_t framecount, int channelnumber, int bandnumber, string name, double min, double max,int numbins)
{

  if(mDebug == mDebugAll)
    {
      cout<<"in createhistogram"<<endl;
      //cout<<"framecount = "<<framecount<<endl;
    }

  min = .999 * min;  
  max = 1.001 * max;
  cout<<"after min"<<endl;
  
  TH1F *histogram = new TH1F("histogram",name.c_str(),numbins,min,max);
  //fill the histo
  if(framecount<1000)
    {
      for(size_t i=0;i<Channels[channelnumber].Bands[bandnumber].mean_vec.size();++i)
	{
	  histogram->Fill(Channels[channelnumber].Bands[bandnumber].mean_vec[i],1);
	  if(mDebug ==mDebugAll)
	    {
	      cout<<setw(20)<<i<<setw(20)<<Channels[channelnumber].Bands[bandnumber].mean_vec[i]<<endl;
	    }
	}
    }else if(framecount>=1000){
      size_t start = framecount -1000;
      for(size_t i = start;i<Channels[channelnumber].Bands[bandnumber].mean_vec.size();++i)
	{
	  histogram->Fill(Channels[channelnumber].Bands[bandnumber].mean_vec[i],1);
	  if(mDebug ==mDebugAll)
	    {
	      cout<<setw(20)<<i<<setw(20)<<Channels[channelnumber].Bands[bandnumber].mean_vec[i]<<endl;
	    }
	}
    }
  if(mDebug == mDebugAll){      cout<<"End of CreateHistogram"<<endl;    }

  histogram->GetYaxis()->CenterTitle();
  histogram->GetXaxis()->CenterTitle();
  histogram->GetYaxis()->SetTitleOffset(1.7);
  histogram->GetYaxis()->SetTitleSize(0.03);
  histogram->GetYaxis()->SetLabelOffset(0);
  histogram->GetYaxis()->SetLabelSize(0.03);
  histogram->GetXaxis()->SetNdivisions(9);

  return histogram;
}//END CREATEHISTOGRAM

void SpecMon::BackgroundSocket(size_t framecount)
{
  int bandnumber =0;
  int channelnumber =0;
  
  TMessage mess(kMESS_OBJECT);
  char str[32];
  socket = new TSocket("localhost",9098);//declared in library file
  socket->Recv(str,32);                  //check to see if the gui needs and update --------------------------------- Socket 1<-
  
  
  if(strcmp(str,"update")==0 || strcmp(str,"testbandupdate")==0)
    {
      if(strcmp(str,"update")==0)
	{
	  if(mDebug == mDebugAll)
	    {
	      cout<<"received update and sending go Socket 1<- & 2->"<<endl;
	    }
	  
	  if(mTestBand)
	    {
	      mTestBand = false;
	      DeleteBand(mChannelTest);//these values must be set in the .hh file
	    } 
	  
	}else if(strcmp(str,"update") != 0){
	  mTestBand = true;//this should be a global variable
	  
	}

      socket->Send("go");            //tell gui that bg is waiting for the info --------------------------------- Socket 2->

      TMonitor *mon = new TMonitor;
      mon->Add(socket);
      TMessage *message;
      TSocket *s;
      s = mon->Select();
      //socket->Recv(message);
      s->Recv(message);                     //get message with #bins, channelnumber, bandnumber, ------------------------ Socket 3<-
      //this is a histogram right now but I want to change to a TBuffer
      
      if(message->What() == kMESS_OBJECT)
	{
	  TH1 *info = (TH1 *)message->ReadObject(message->GetClass());
	  guibins = (int)info->GetBinContent(1);
	  channelnumber = (int)info->GetBinContent(2);
	  bandnumber = (int)info->GetBinContent(3);
	  if(mDebug == mDebugAll)
	    {
	      cout<<"channelnumber = " <<channelnumber<<endl;
	      cout<<"bandnumber = "<<bandnumber<<endl;
	    }

	  delete info;			
	  mon->Remove(s);
	}else{
	  throw runtime_error("Error with getting the values of channelnumber @ bg Socket 3");
	}
      
      delete message;
      socket->Send("ready");                //send a ready to start tranmitting data ------------------------------------ Socket 4->
      socket->Recv(str,32);                 //receive the ready signal from gui ----------------------------------------- Socket 5<-
    
      if(strcmp(str,"ready") ==0)
	{
	  
	  string StripTitle = Channels[channelnumber].Bands[bandnumber].BandName + " 1000 second Strip chart ";
	  if(mTestBand)
	    {
	      framecount = mFrameCount_Test;
	    }
	  
	  TH1F *stripchart = CreateStripChart(framecount, channelnumber, bandnumber,StripTitle, Channels[channelnumber].Bands[bandnumber].minPower,Channels[channelnumber].Bands[bandnumber].maxPower);;
	  socket->Send("stripchart");  //send message that strip chart is ready ------------------------------------- Socket 6->  
	  socket->Recv(str,32);        //get confirm ---------------------------------------------------------------- Socket 7<-

	  if(strcmp(str,"ready")==0)
	    {
	      //everything is fine
	    }else{
	      throw runtime_error("Error in socket when sending string of stripchart");
	    }
	  
	  mess.WriteObject(stripchart);
	  socket->Send(mess);          //send the stripchart -------------------------------------------------------- Socket8->
	  delete stripchart;
	}
      string HistTitle = Channels[channelnumber].Bands[bandnumber].BandName + " Histogram of Integrated Power ";
      TH1F *histogram = CreateHistogram(framecount,channelnumber,bandnumber,HistTitle,Channels[channelnumber].Bands[bandnumber].minPower,Channels[channelnumber].Bands[bandnumber].maxPower,guibins);
      
      socket->Send("histogram");          //send a message histogram is ready ------------------------------------------- Socket 9->
      socket->Recv(str,32);               //get confirm =---------------------------------------------------------------- Socket 10<-
      if(strcmp(str,"ready")==0)
	{
	  //everything is fine
	}else{
	  cout<<"Error sending  histogram"<<endl;
	  throw runtime_error("Error in socket when sending string of histo");
	}
      mess.Reset();
      mess.WriteObject(histogram);
      socket->Send(mess);                 //send the histogram ---------------------------------------------------------- Socket 11->
      delete histogram;
      
      TH1S *mean_hist;
      mean_hist =  new TH1S("means","Means of power and triggers",21,0,21);
      FillMeanHistogram(mean_hist,framecount,bandnumber,channelnumber);
      
      socket->Send("mean_values");       //send message mean values are ready ------------------------------------------- Socket 12->
      socket->Recv(str,32);              //get confirm ------------------------------------------------------------------ Socket 13<-
      if(strcmp(str,"ready") ==0)
	{
	  //everything is fine
	}else{
	  throw runtime_error("Error in socket when sending mean values");
	}
      mess.Reset();
      mess.WriteObject(mean_hist);
      socket->Send(mess);                //send the mean values ---------------------------------------------------------- Socket 14->
      delete mean_hist;
      
      socket->Send("Finished");          //send that the process is done ------------------------------------------------- Socket 15->
      //      socket->Close();
      // delete socket;
      delete mon;
    }else if(strcmp(str,"testbandupdatefirst")==0){
      
      double newfreqlo =0;
      double newfreqhi =0;
      double newpowerlo =0;
      double newpowerhi =0;

      mFrameCount_Test = 0;
     
      socket->Send("go");            //tell gui that bg is waiting for the info --------------------------------- Socket 2t->
 
      TMonitor *mon = new TMonitor; 
      mon->Add(socket);
      TMessage *message;
      TSocket *s;
      s = mon->Select();
      s->Recv(message);              //get the new info for the new band----------------------------------------- Socket 3t<-
      //socket->Recv(message);

      if(message->What() == kMESS_OBJECT)
	{
	  
	  TH1* newband = (TH1 *) message->ReadObject(message->GetClass());
	  newfreqlo = (double)newband->GetBinContent(1);
	  newfreqhi = (double)newband->GetBinContent(2);
	  newpowerlo = (double)newband->GetBinContent(3);
	  newpowerhi = (double)newband->GetBinContent(4);
	  mChannelTest = (int)newband->GetBinContent(5);
	  mBandTest = (int)newband->GetBinContent(6);
	  delete newband;
	}
      int bandnum = Channels[mChannelTest].BandNum;
      if(Channels[mChannelTest].Bands[(bandnum-1)].BandName == "Test Band")
	{
	  DeleteBand(mChannelTest);
	}
      mTestBand = true;
      AddBand(mChannelTest,mBandTest,newfreqlo,newfreqhi,newpowerlo,newpowerhi);     
      s->Close();
      mon->Remove(s);
    }
  
  socket->Close();
  delete socket;
  //cout<<"socket is closed"<<endl;
    
  return;
  
}//END BACKGROUNDSOCKET

void SpecMon::AddBand(int channelnumber , int bandnumber, double freqlo, double freqhi, double powerlo, double powerhi){

  BandInfo temp;
  temp.freqlo = freqlo;
  temp.freqhi = freqhi;
  temp.powerlo = powerhi;
  temp.powerhi = powerhi;
  temp.BandName = "Test Band";

  Channels[channelnumber].Bands.insert(Channels[channelnumber].Bands.end(),temp);
  ++Channels[channelnumber].BandNum;
  mChannelTest = channelnumber;
  mBandTest = bandnumber;
  
  mTestBand =true;
  
}//END ADDBAND

void SpecMon::DeleteBand(int channelnumber){

  Channels[channelnumber].Bands.pop_back();
  Channels[channelnumber].BandNum = (Channels[channelnumber].BandNum -1);

}//END DELETEBAND

/********************************************
*********************************************
**                                         **
**              ProcessData                ** 
**                                         **
*********************************************
********************************************/
void SpecMon::ProcessData()
{
  //cout<<"in process data"<<endl;
  Time now=mIn.getFillTime();
  if(mFrameCount ==5)
    {
      cout<<"Time is "<<now<<endl;
    }

  string eObj;
  if(mFrameCount == 1 && mDebug == mDebugAll)
    {
      cout<<"frame = "<<mFrameCount<<endl;
    }

  //fork stuff whooooooo, this creates a new root mainwindow to display the spectrum of the selected band
  
  /*  
      pid_t pid;
  if(mFrameCount == 6 &&  (pid = fork()) != 0)
    {
      cout<<"forking "<<endl;
      mFork = true;
    }else if(pid == 0 && mFrameCount == 6){
      char *env_init[] = {"USER=unknown", "PATH=/export/home/bodtim/GuiMonTest/testing",NULL};
      if(execv("/export/home/bodtim/GuiMonTest/testing/SpectrumGui",env_init)<0)
	{
	  cout<<" There are problems with exec()"<<endl;
	}
    }
  
  */
  
  // int bandnumber=0;
  // int channelnumber=0;
  
  //Socket is created every time a frame is read in after frame 100 so that the strip chart and histogram on the gui will be filled
  //the socket checks for is the gui wants an update.  or if there is a change that must be made through the gui
	     	
  if(mFrameCount>5)
    {
      BackgroundSocket(mFrameCount);
    }
		  
  ++mFrameCount;
  ++mFrameCount_Test;

  /*************************************
  *change this into and exec loop      * 
  *************************************/
  
  //create the initial gui
  if (mFrameCount == 5) 
    {
      //      cout<<"before creation mFrameCount = "<<mFrameCount<<endl;	
      
      // get channel names
      string chnnames;
      chnnames = "H2:LSC-AS_Q 16384 H1:LSC-AS_Q 16384";
      // create main window
      //cout<<"whats up"<<endl;
      fMainWindow = new SpecMonWindow (gClient->GetRoot(), 900, 500,
				       *this, chnnames.c_str());
      fMainWindow->Resize (TGDimension (1000, 700));
      // create GUI thread
      pthread_t		tid;
      pthread_attr_t	tattr;
      struct sched_param	schedprm;
      int			status;
      // set thread parameters: detached & process scope
      if (pthread_attr_init (&tattr) != 0) {
	cerr << "Unable to create GUI thread" << endl;
	exit (0);
      }
      pthread_attr_setdetachstate (&tattr, PTHREAD_CREATE_DETACHED);
      pthread_attr_setscope (&tattr, PTHREAD_SCOPE_PROCESS);
      // set low priority
      pthread_attr_getschedparam (&tattr, &schedprm);
      schedprm.sched_priority = 10;
      pthread_attr_setschedparam (&tattr, &schedprm);
      // create thread
      status = pthread_create (&tid, &tattr, guithread, 0);
      pthread_attr_destroy (&tattr);
      if (status != 0) {
	cerr << "Unable to create GUI thread" << endl;
	exit (0);
      }
      //fFirstTime = false;
    }
   
  //Start of Data Processing
  
  if(mDebug == mDebugAll)
    {
      cout<<"Time "<<now<<endl;
      cout <<"mFrameCount "<<mFrameCount<<endl;
    }
  
  mTotalRuntime += getDacc().getStride();  
  
  vector<FSeries> mFSeries;
  ostream *dLog;
  int dnumber =0;
  
  //cout<<"before channel loop"<<endl;

  //______________________________________________Channel Loop Process Data___________________________
  for(int i=0; i<mChannelNumber;++i)
    {
      mTSeries = getDacc().refData(Channels[i].ChannelName.c_str());

      if(!mTSeries)
	{
	  cout<<"There is no data"<<endl;
	}  	
  	
      mFSeries_temp = new FSeries;
      *mFSeries_temp = FSeries(*mTSeries);
      mFSeries.insert(mFSeries.end(),*mFSeries_temp);
      delete mFSeries_temp;
      mFSeries[i].setData(*mTSeries);
      FSeries& mFS(mFSeries[i]);  

      /*      
      if(mFrameCount%30 ==0 && channelnumber == i)
	{
	  //cout<<"in specmon to spectrum socket loop"<<endl;	 
	  socket = new TSocket("localhost", 9099);
	  char str[32];
	  socket->Recv(str,32);
	  if(strcmp(str,"go")==0)
	    {
	      cout<<"specmon sending go to spectrum"<<endl;
	      socket->Send("go");
	      TMessage mess2(kMESS_OBJECT);
	      
	      //create histogram of power	     
	      TH1D *hist;     
	      
	      FSeries fs;
	      fs = mFSeries_temp->extract(Channels[i].Bands[bandnumber].freqlo,(Channels[i].Bands[bandnumber].freqhi-Channels[i].Bands[bandnumber].freqlo));
	      hist = new TH1D("power","Power Spectrum",guibins,Channels[i].Bands[bandnumber].freqlo, Channels[i].Bands[bandnumber].freqhi);
	      	      
	      float* data;
	      data = (float*)fs.refData();
	      
	      for(int l =Channels[i].Bands[bandnumber].freqlo ; l<Channels[i].Bands[bandnumber].freqhi;++l)
		{
		  float temp = *data;
		  float number = (temp * temp);
		  hist->SetBinContent(l,number);
		  ++data;		  
		}
	      mess2.WriteObject(hist);
	      socket->Send(mess2);
	      //cout<<"sent histo of power spec to spectrum"<<endl;	            
	    }	  
	}
      */

      //cout<<"after the new socket loop"<<endl;
      	
	//_____________________________________Parseval's  Check_________________________________________________
	
      if(mDebug == mDebugParseval || mDebug ==mDebugAll)
	{
	  float pFreqPower=0;
	  pFreqPower = mFS.Power(); //sums from min to max freq sampled if no freq limits are specified(0 to Nyquist)
	  //cout<<"Parsevals = "<<pFreqPower<<endl;
		
	  //get (rms)^2 of time series or get summation from 1 to n (the number of data samples ) value squared divided by n
	  	
	  size_t pNSample = mTSeries->getNSample();//number of sample o
	  //cout<<"# of samples "<<pNSample<<endl;
	  	
	  Interval pTStep = mTSeries->getTStep();//gives the time step or 1/pNSample seconds
	  //cout<<"TStep = "<<pTStep<<endl;

	  //couldnt figure out a better way to do this.  i was unable to access individual bins of the TSeries from Dacc
	  	
	  TSeries *pTSeries;	
	  pTSeries = new TSeries;
	  	
	  double pPower=0;		//RMS^2
	  Time pTime = mTSeries->getStartTime();	//beginning of original TSeries
	  //cout<<"Start time = "<<pTime<<endl;

	  //go through the original TSeries and then make a TSeries of one bin.  Average it to get the value and square it
	  for(size_t parseval=0;parseval<(pNSample+1);++parseval)
	    {
	      *pTSeries = mTSeries->extract(pTime,pTStep);
	      pTime = (pTime + pTStep);
	  		
	      double pPower_temp =0;
	      pPower_temp = pTSeries->getAverage();
	      double pPower_temp1 =(pPower_temp * pPower_temp);

	      pPower = (pPower + pPower_temp1);
	    }//

	  delete pTSeries;
      	  pPower = (pPower/(pNSample));
	  
	  //cout<<"RMS^2 = "<<pPower<<endl;
	  	
	}	

      //___________________________________________End of Parsevals Test___________________________________	

      //cout<<"after parsevals"<<endl;
	
      //-------------------------------------BAND LOOP---------------------------------------------------
  	
      for(int a=0;a<Channels[i].BandNum;++a)
	{	
	  float freqlo = Channels[i].Bands[a].freqlo;//mBand_temp[a].freqlo;
	  float freqhi = Channels[i].Bands[a].freqhi;//mBand_temp[a].freqhi;
	  float BandPower = mFS.Power(freqlo,freqhi);
		
	  //---------------------------------------Computation of Average Power for 24,12,6,1 hour periods-----------------
	  vector<float> mean_vec_temp; //temp vector to hold mean_vec for insertion and deletion
	
	  size_t framecount;
	  if(mTestBand && i == mTestChannel && a == (Channels[mTestChannel].BandNum -1))
	    {
	      framecount = mFrameCount_Test;
	    }else{
	      framecount = mFrameCount;
	    }

	  if(framecount<= _24average)
	    {
	      float totalpower=0;
	      //couldnt figure out a better way to insert elements into mean_vec.  kept getting udeclared mean_vec error.
		
	      mean_vec_temp = Channels[i].Bands[a].mean_vec;
	      mean_vec_temp.insert(mean_vec_temp.end(),BandPower);
	      Channels[i].Bands[a].mean_vec = mean_vec_temp;

	      if(BandPower>Channels[i].Bands[a].maxPower)//check and see if the power is at its highest pt of run, this will be used when computing the histogram for the gui
		{
		  Channels[i].Bands[a].maxPower = BandPower;
		}else if(BandPower<Channels[i].Bands[a].minPower){
		 
		  Channels[i].Bands[a].minPower = BandPower;
		}
	      
	      if(mFrameCount ==1)
		{
		  Channels[i].Bands[a].minPower = BandPower;
		}
	      
	      //this computation of the average must be changed, take the mean from before and then times that my (mFrameCount -1) 
	      //then add the new power and divide by mFrameCount
	      
	      if(framecount<= (_24average/2))
		{		  
		  if(framecount <= (_24average/4))
		    {		      
		      if(framecount<= (_24average/24))
			{
			  if(framecount<= (_24average/48))
			    {
			      if(framecount<= (_24average/144))
				{				  
				  if(framecount<= (_24average/1440))
				    {
				      //do nothing want means to be 0
				    }else if(framecount > (_24average/1440)){
				      
				      size_t start = framecount-60;
				      
				      for(size_t c= start;c<framecount;++c)
					{
					  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
					  totalpower = totalpower + totalpower_temp;
					}
				      
				      Channels[i].Bands[a]._1min_mean = totalpower/60;
				    }
					     
				}else if(framecount>(_24average/144)){
				  
				  size_t start = framecount-60;

				  for(size_t c =start;c<framecount;++c)
				    {
				      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
				      totalpower = totalpower + totalpower_temp;
				    }
				  Channels[i].Bands[a]._1min_mean = totalpower/60;

				  size_t finish = start;
				  start = framecount -600;
				  for(size_t c = 0;c<finish;++c)
				    {
				      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
				      totalpower = totalpower + totalpower_temp;
				    }
				  Channels[i].Bands[a]._10min_mean = totalpower/600;
				}

			    }else if(framecount>(_24average/48)){
			      
			      size_t start = framecount-60;
			      
			      for(size_t c =start;c<framecount;++c)
				{
				  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
				  totalpower = totalpower + totalpower_temp;
				}
			      Channels[i].Bands[a]._1min_mean = totalpower/60;
			      
			      size_t finish = start;
			      start = framecount -600;
			      
			      for(size_t c = start;c<finish;++c)
				{
				  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
				  totalpower = totalpower + totalpower_temp;
				}
			      Channels[i].Bands[a]._10min_mean = totalpower/600; 

			      finish =start;
			      start =framecount - 1800;
			      
			      for(size_t c = start;c<finish;++c)
				{
				  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
				  totalpower = totalpower + totalpower_temp;
				}

			      Channels[i].Bands[a]._30min_mean = totalpower/1800; 
			      
			    }

			}else if(framecount>(_24average/24)){
			  
			  size_t start = framecount-60;
			  
			  for(size_t c =start;c<framecount;++c)
			    {
			      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
			      totalpower = totalpower + totalpower_temp;
			    }
			  Channels[i].Bands[a]._1min_mean = totalpower/60;
			  
			  size_t finish = start;
			  start = framecount -600;
			  
			  for(size_t c = start;c<finish;++c)
			    {
			      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
			      totalpower = totalpower + totalpower_temp;
			    }
			  Channels[i].Bands[a]._10min_mean = totalpower/600; 
			  
			  //finish =start;
			  start =framecount-1800;
			  
			  for(size_t c = start;c<framecount;++c)
			    {
			      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
			      totalpower = totalpower + totalpower_temp;
			    }
			  
			  Channels[i].Bands[a]._30min_mean = totalpower/1800; 
			  
			  //finish =start;
			  start =framecount-3600;
			  
			  for(size_t c = start;c<framecount;++c)
			    {
			      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
			      totalpower = totalpower + totalpower_temp;
			    }
			  
			  Channels[i].Bands[a]._1hr_mean = totalpower/3600; 
			}

		    }else if(framecount>(_24average/4)){
		      
		      size_t start = framecount-60;
		      
		      for(size_t c =start;c<framecount;++c)
			{
			  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
			  totalpower = totalpower + totalpower_temp;
			}
		      Channels[i].Bands[a]._1min_mean = totalpower/60;
		      
		      size_t finish = framecount;
		      start = framecount -600;
		      
		      for(size_t c = start;c<finish;++c)
			{
			  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
			  totalpower = totalpower + totalpower_temp;
			}
		      Channels[i].Bands[a]._10min_mean = totalpower/600; 
		      
		      finish =framecount;
		      start =framecount-1800;
		      
		      for(size_t c = start;c<finish;++c)
			{
			  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
			  totalpower = totalpower + totalpower_temp;
			}
		      
		      Channels[i].Bands[a]._30min_mean = totalpower/1800; 
		      
		      //finish =start;
		      start =framecount-3600;
		      
		      for(size_t c = start;c<finish;++c)
			{
			  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
			  totalpower = totalpower + totalpower_temp;
			}
		      
		      Channels[i].Bands[a]._1hr_mean = totalpower/3600; 
		      
		      //finish =start;
		      start =framecount-(_24average/4);

		      for(size_t c = start;c<finish;++c)
			{
			  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
			  totalpower = totalpower + totalpower_temp;
			}
		      
		      Channels[i].Bands[a]._6hr_mean = totalpower/(_24average/4); 
		      
		    }
		}else if(framecount>(_24average/2)){

		  size_t start = framecount-60;
		  
		  for(size_t c =start;c<framecount;++c)
		    {
		      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		      totalpower = totalpower + totalpower_temp;
		    }
		  Channels[i].Bands[a]._1min_mean = totalpower/60;
		  
		  size_t finish = framecount;
		  start = framecount -600;
		  
		  for(size_t c = start;c<finish;++c)
		    {
		      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		      totalpower = totalpower + totalpower_temp;
		    }
		  Channels[i].Bands[a]._10min_mean = totalpower/600; 
		  
		  //finish =start;
		  start =framecount-1800;
		  
		  for(size_t c = start;c<finish;++c)
		    {
		      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		      totalpower = totalpower + totalpower_temp;
		    }
		  
		  Channels[i].Bands[a]._30min_mean = totalpower/1800; 
		  
		  //finish =start;
		  start =framecount-3600;
		  
		  for(size_t c = start;c<finish;++c)
		    {
		      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		      totalpower = totalpower + totalpower_temp;
		    }
		  
		  Channels[i].Bands[a]._1hr_mean = totalpower/3600; 
		  
		  //finish =start;
		  start =framecount-(_24average/4);
		  
		  for(size_t c = start;c<finish;++c)
		    {
		      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		      totalpower = totalpower + totalpower_temp;
		    }
		  
		  Channels[i].Bands[a]._6hr_mean = totalpower/(_24average/4);
		  
		  // finish = start;
		  start = framecount - (_24average/2);
		  
		  for(size_t c = start;c<finish;++c)
		    {
		      float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
			  totalpower = totalpower + totalpower_temp;
		    }
		  Channels[i].Bands[a]._12hr_mean = totalpower/(_24average/2);		  
		}

	    }else if(framecount > _24average){
	      
	      float totalpower =0;
	      size_t start = framecount-60;
	      
	      for(size_t c =start;c<framecount;++c)
		{
		  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		  totalpower = totalpower + totalpower_temp;
		}
	      Channels[i].Bands[a]._1min_mean = totalpower/60;
	      
	      size_t finish = framecount;
	      start = framecount -600;
	      
	      for(size_t c = start;c<finish;++c)
		{
		  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		  totalpower = totalpower + totalpower_temp;
		}
	      Channels[i].Bands[a]._10min_mean = totalpower/600; 
	      
	      //finish =start;
	      start =framecount-1800;
	      
	      for(size_t c = start;c<finish;++c)
		{
		  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		  totalpower = totalpower + totalpower_temp;
		}
	      
	      Channels[i].Bands[a]._30min_mean = totalpower/1800; 
	      
	      //finish =start;
	      start =framecount-3600;
	      
	      for(size_t c = start;c<finish;++c)
		{
		  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		  totalpower = totalpower + totalpower_temp;
		}
	      
	      Channels[i].Bands[a]._1hr_mean = totalpower/3600; 
	      
	      //finish =start;
	      start =framecount-(_24average/4);
	      
	      for(size_t c = start;c<finish;++c)
		{
		  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		  totalpower = totalpower + totalpower_temp;
		}
	      
	      Channels[i].Bands[a]._6hr_mean = totalpower/(_24average/4);
	      
	      //finish = start;
	      start = framecount - (_24average/2);
	      	      
	      for(size_t c = start;c<finish;++c)
		{
		  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		  totalpower = totalpower + totalpower_temp;
		}
	      
	      Channels[i].Bands[a]._12hr_mean = totalpower/(_24average/2);
	      
	      //finish = start;
	      start = framecount -_24average;
	      
	      for(size_t c = start;c<finish;++c)
		{
		  float totalpower_temp = Channels[i].Bands[a].mean_vec[c];
		  totalpower = totalpower + totalpower_temp;
		}
	      
	      Channels[i].Bands[a]._24hr_mean = totalpower/_24average;	      
	      
	    }

//-------------------------------------------------End of Computation Portion----------------------------------------	
	
	  if(mDebug==-9 || mDebug>1)
	    {
	      cout<<"Power in "<<Channels[i].ChannelName<<" between "<<freqlo<<" and "<<freqhi<<" at time "<<now<<" is "<<BandPower<<endl;
	    }
	  
	  ////////////////////
	  ///////////////////////////
	   
	  //cout<<"before checking band power"<<endl; 
	  //cout<<i<<" "<<a<<endl;
	  if(BandPower>Channels[i].Bands[a].powerhi)
	    {
	      Channels[i].Bands[a].hi_status = true;
	      Channels[i].Bands[a].lo_status = false;
	      
	      //change the order of the checks for below
	      
	      if((mFrameCount ==1 || (mFrameCount - Channels[i].Bands[a].LastProblemFrame_hi) >= 15))
		{
		  BandProblemActions(Channels,i,a,now,mTestBand,Channels[i].Bands[a].triggers,Channels[i].Bands[a].trigger_times,Channels[i].Bands[a].trigger_times_t);
		  if(mDebug == mDebugAll)
		    {
		      cout<<"Do BandActions"<<endl;
		    }
		  Channels[i].Bands[a].triggers.insert(Channels[i].Bands[a].triggers.end(),3);
		  Channels[i].Bands[a].trigger_times.insert(Channels[i].Bands[a].trigger_times.end(),mFrameCount);
		  Channels[i].Bands[a].LastProblemFrame_hi = mFrameCount;
		  Channels[i].Bands[a].LastProblemTime_hi = now;		  
		}
	    }else if(BandPower<Channels[i].Bands[a].powerlo){

	      Channels[i].Bands[a].lo_status = true;
	      Channels[i].Bands[a].hi_status = false;
		  
	      if((mFrameCount ==1 || (mFrameCount - Channels[i].Bands[a].LastProblemFrame_lo) >= 15))
		{
		  if(mDebug == mDebugAll)
		    {
		      cout<<"Do low BandActions for Channel "<<i<<" and Band "<<a<<endl;
		    }
		  BandProblemActions(Channels,i,a,now,mTestBand,Channels[i].Bands[a].triggers,Channels[i].Bands[a].trigger_times,Channels[i].Bands[a].trigger_times_t);
		  
		  Channels[i].Bands[a].triggers.insert(Channels[i].Bands[a].triggers.end(),5);
		  Channels[i].Bands[a].trigger_times.insert(Channels[i].Bands[a].trigger_times.end(),mFrameCount);
		  Channels[i].Bands[a].LastProblemFrame_lo = mFrameCount;
		  Channels[i].Bands[a].LastProblemTime_lo = now;

		}

	    }else if(BandPower<=Channels[i].Bands[a].powerhi && BandPower >=Channels[i].Bands[a].powerlo){
	      Channels[i].Bands[a].hi_status = false;
	      Channels[i].Bands[a].lo_status = false;
	    }

	    //Compute trigger stats--------------------------------------------------------------------------------------------------------
	    //need to find the place to start the search for the mean....

	  //int trigger_lo=0;
	  //int trigger_hi=0;
	  //	  int LastProblemFrame_hi = Channels[i].Bands[a].LastProblemFrame_hi;
	  // int LastProblemFrame_lo = Channels[i].Bands[a].LastProblemFrame_lo;
	
	  // cout<<"LastProblemFrame_hi = "<<LastProblemFrame_hi<<endl;

	  //cout<<"LastProblemFrame_lo = "<<LastProblemFrame_lo<<endl;

	  //put in new trigger stuff

	  //*********************************************************************************************
	  ///this stuff on the triggers needs to be finished and checked over//////
	  //************************************************************************************
	  if(mTestBand == false)
	    {
	      //open logfile for specific band with append mode
	      dLogfile.open(dLogfilename[dnumber].c_str(),ios::app);
	      
	      if(!dLogfile)
		{
		  cerr<<"Couldn't open logfile for specific band "<<Channels[i].Bands[a].BandName<<endl;
		  cerr<<"       reverting to stdout"<<endl;
		  dLog = &std::cout;
		}else
		  {
		    dLog = &dLogfile;
		  }
	      
	      //cout<<"before logging"<<endl;
	      if(mFrameCount ==1)
		{
		  *dLog<<"###############################################################################################"<<endl;
		  *dLog<<"SpecMon started at time "<<now<<endl<<endl;
		  //*dLog<<setw(20)<<"Time"<<setw(20)<<"Power"<<setw(20<<"1hr mean"<<setw(20)<<"6hr mean"<<setw(20)<<"12 hr mean"<<setw(20)<<"24hr mean"<<endl;
		  
		}
	      
	      if(mDebug == mDebugAll && mFrameCount%15 != 0)
		{
		  *dLog<<setw(20)<<now<<setw(20)<<BandPower<<endl;
		  //*dLog<<BandPower<<endl;
		}
	      
	      if(mFrameCount%15 ==0 || mFrameCount == 1)
		{
		  if(mTestBand && (Channels[i].BandNum - a) == 1)
		    {
		    }else{
		      
		      *dLog<<setw(20)<<now<<setw(20)<<BandPower<<setw(20)<<Channels[i].Bands[a]._1hr_mean<<setw(20)<<Channels[i].Bands[a]._6hr_mean<<setw(20)<<Channels[i].Bands[a]._12hr_mean<<setw(20)<<Channels[i].Bands[a]._24hr_mean<<endl;
		    }  
		}
	      
	      dLogfile.close();
	      ++dnumber; 		
	    }
	}
    }

  if (mDebug > 0)
    {
      *mLog << endl << mName << ": Entered ProcessData() with mFrameCount = "
	    << mFrameCount << endl;
    }
  //cout<<"befor tseries delete"<<endl;
  //delete mTSeries;   
  return;	
}

//END PROCESSDATA   

BandInfo::BandInfo()
{
  BandName = " ";
  freqlo = 0;
  freqhi = 0;
  powerlo=0;
  powerhi=0;
  BandErrorCount_lo = 0;
  BandErrorCount_hi =0;
  LastProblemTime_lo = 0;
  LastProblemTime_hi =0;
  LastProblemFrame_lo =0;
  LastProblemFrame_hi =0;
  AlarmDesc = " ";
  hi_status = false;
  lo_status = false;
  _24hr_mean =0;
  _12hr_mean = 0;
  _6hr_mean =0;
  _1hr_mean =0;
  _1min_mean =0;
  _10min_mean = 0;
  _30min_mean = 0;
  trigger24_lo = 0;
  trigger12_lo =0;
  trigger6_lo =0;
  trigger1_lo=0;
  trigger30min_lo =0;
  trigger10min_lo =0;
  trigger1min_lo =0;
  trigger24_hi = 0;
  trigger12_hi =0;
  trigger6_hi =0;
  trigger1_hi=0;
  trigger30min_hi =0;
  trigger10min_hi =0;
  trigger1min_hi = 0;
  maxPower =0;
  minPower =0;
}

BandInfo::~BandInfo()
{
  //int Debug =0;
  //if(Debug ==5 || Debug >10)
  //cout<<"Band this does nothing right now"<<endl;
}

ChannelInfo::ChannelInfo()
{
  BandNum = 0;
  ChannelName = " ";
}

ChannelInfo::~ChannelInfo()
{
  //int Debug =0;  //variables to store the power in the Bands
  // _____________________________________________________________________
  //vector<float> mBand_power;
  //if(Debug ==5 || Debug >10)
  //cout<< "Channel this does nothing right now,"<<endl;
}
