#include "StrainGet.hh"

extern "C" {
#include <lal/AVFactories.h>
};

StrainGet::StrainGet( StrainGetArguments *args ) {
	firstRun = true;

	inputData.Go.re = args->G0Re;
	inputData.Go.im = args->G0Im;
	inputData.Do.re = args->D0Re;
	inputData.Do.im = args->D0Im;
	inputData.Wo.re = args->W0Re;
	inputData.Wo.im = args->W0Im;
	inputData.f = args->f;
	inputData.To = args->To;
	
	inputData.AS_Q.epoch.gpsNanoSeconds = inputData.DARM.epoch.gpsNanoSeconds
		= inputData.EXC.epoch.gpsNanoSeconds 
		= inputData.DARM_ERR.epoch.gpsNanoSeconds = 0;
	
	inputData.wings = args->wings;
	inputData.usefactors = inputData.fftconv = inputData.outalphas = inputData.delta = 1;
		
	readFiltersFile();
}

void StrainGet::computeStrain( long dataStartTime, double deltaT, int length,
	double asq[], double darm[], double darm_err[], double exc[], double **h, 
	double **hC, double **hR, double **alpha, double **beta, double **alphabeta ) 
{
	LALStatus status;
	int duration = (int)(deltaT * length);

/*	if( !firstRun ) {
		LALDDestroyVector(&status,&outputData.h.data);
		LALDDestroyVector(&status,&outputData.hC.data);
		LALDDestroyVector(&status,&outputData.hR.data);
		LALZDestroyVector(&status,&outputData.alpha.data);
		LALZDestroyVector(&status,&outputData.beta.data);
		LALZDestroyVector(&status,&outputData.alphabeta.data);

		LALSDestroyVector(&status,&inputData.AS_Q.data);
		LALSDestroyVector(&status,&inputData.DARM.data);
		LALSDestroyVector(&status,&inputData.DARM_ERR.data);
		LALSDestroyVector(&status,&inputData.EXC.data);
	}
*/
	if( firstRun ) {
		LALSCreateVector(&status, &inputData.AS_Q.data, length );
		LALSCreateVector(&status, &inputData.DARM.data, length );
		LALSCreateVector(&status, &inputData.EXC.data, length );
		LALSCreateVector(&status, &inputData.DARM_ERR.data, length );
	}

	inputData.AS_Q.epoch.gpsSeconds = inputData.DARM.epoch.gpsSeconds
		= inputData.EXC.epoch.gpsSeconds = inputData.DARM_ERR.epoch.gpsSeconds
		= dataStartTime;
	inputData.AS_Q.deltaT = inputData.DARM.deltaT = inputData.DARM_ERR.deltaT
		= inputData.EXC.deltaT = deltaT;
	
	for( int i = 0; i < length; i++ ) {
		inputData.AS_Q.data->data[i] = asq[i];
		inputData.DARM.data->data[i] = darm[i];
		inputData.DARM_ERR.data->data[i] = darm_err[i];
		inputData.EXC.data->data[i] = exc[i];
	}
	
	outputData.h.epoch = outputData.hC.epoch = outputData.hR.epoch
		= outputData.alpha.epoch = outputData.alphabeta.epoch
		= outputData.beta.epoch = inputData.AS_Q.epoch;
	outputData.h.deltaT = outputData.hC.deltaT = outputData.hR.deltaT
		= inputData.AS_Q.deltaT;
	outputData.alpha.deltaT = outputData.alphabeta.deltaT 
		= outputData.beta.deltaT = inputData.To;

	if( firstRun ) {
		LALDCreateVector( &status, &outputData.h.data,
			(UINT4)(duration/outputData.h.deltaT +0.5) );
		LALDCreateVector( &status, &outputData.hC.data,
			(UINT4)(duration/outputData.hC.deltaT +0.5) );
		LALDCreateVector( &status, &outputData.hR.data,
			(UINT4)(duration/outputData.hR.deltaT +0.5) );
		
		LALZCreateVector( &status, &outputData.alpha.data,
			(UINT4)(duration/outputData.alpha.deltaT +0.5) );
		LALZCreateVector( &status, &outputData.alphabeta.data,
			(UINT4)(duration/outputData.alphabeta.deltaT +0.5) );
		LALZCreateVector( &status, &outputData.beta.data,
			(UINT4)(duration/outputData.beta.deltaT +0.5) );
	}
	LALComputeStrainDMT( &status, &outputData, &inputData );

	*h = &outputData.h.data->data[0];
	*hC = &outputData.hC.data->data[0];
	*hR = &outputData.hR.data->data[0];
	*alpha = &outputData.alpha.data->data[0].re;
	*beta = &outputData.beta.data->data[0].re;
	*alphabeta = &outputData.alphabeta.data->data[0].re;

	firstRun = false;
}

StrainGet::~StrainGet() {
	LALStatus status;

	LALDDestroyVector(&status,&outputData.h.data);
	LALDDestroyVector(&status,&outputData.hC.data);
	LALDDestroyVector(&status,&outputData.hR.data);
	LALZDestroyVector(&status,&outputData.alpha.data);
	LALZDestroyVector(&status,&outputData.beta.data);
	LALZDestroyVector(&status,&outputData.alphabeta.data);

	LALSDestroyVector(&status,&inputData.AS_Q.data);
	LALSDestroyVector(&status,&inputData.DARM.data);
	LALSDestroyVector(&status,&inputData.DARM_ERR.data);
	LALSDestroyVector(&status,&inputData.EXC.data);
}

void StrainGet::readFiltersFile() {

}

